@@ -2,8 +2,7 @@
 
   bignum.c -
 
-  $Author: usa $
-  $Date: 2007-12-25 03:12:24 +0900 (Tue, 25 Dec 2007) $
+  $Author: nobu $
   created at: Fri Jun 10 00:48:55 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -37,7 +36,19 @@ VALUE rb_cBignum;
 #define BIGLO(x) ((BDIGIT)((x) & (BIGRAD-1)))
 #define BDIGMAX ((BDIGIT)-1)
 
-#define BIGZEROP(x) (RBIGNUM_LEN(x) == 0 || (RBIGNUM_LEN(x) == 1 && BDIGITS(x)[0] == 0))
+#define BIGZEROP(x) (RBIGNUM_LEN(x) == 0 || \
+		     (BDIGITS(x)[0] == 0 && \
+		      (RBIGNUM_LEN(x) == 1 || bigzero_p(x))))
+
+static int
+bigzero_p(VALUE x)
+{
+    long i;
+    for (i = 0; i < RBIGNUM_LEN(x); ++i) {
+	if (BDIGITS(x)[i]) return 0;
+    }
+    return 1;
+}
 
 #define RBIGNUM_SET_LEN(b,l) \
   ((RBASIC(b)->flags & RBIGNUM_EMBED_FLAG) ? \
@@ -380,9 +391,9 @@ rb_cstr_to_inum(const char *str, int base, int badcheck)
 
 #define conv_digit(c) \
     (!ISASCII(c) ? -1 : \
-     isdigit(c) ? ((c) - '0') : \
-     islower(c) ? ((c) - 'a' + 10) : \
-     isupper(c) ? ((c) - 'A' + 10) : \
+     ISDIGIT(c) ? ((c) - '0') : \
+     ISLOWER(c) ? ((c) - 'a' + 10) : \
+     ISUPPER(c) ? ((c) - 'A' + 10) : \
      -1)
 
     if (!str) {
@@ -439,7 +450,7 @@ rb_cstr_to_inum(const char *str, int base, int badcheck)
 	len = 2;
 	break;
       case 8:
-	if (str[0] == '0' && (str[1] == 'o'||str[1] == 'O'||str[1] == '_')) {
+	if (str[0] == '0' && (str[1] == 'o'||str[1] == 'O')) {
 	    str += 2;
 	}
       case 4: case 5: case 6: case 7:
@@ -470,8 +481,15 @@ rb_cstr_to_inum(const char *str, int base, int badcheck)
 	}
 	break;
     }
-    if (*str == '0') {		/* squeeze preceeding 0s */
-	while (*++str == '0');
+    if (*str == '0') {		/* squeeze preceding 0s */
+	int us = 0;
+	while ((c = *++str) == '0' || c == '_') {
+	    if (c == '_') {
+		if (++us >= 2)
+		    break;
+	    } else
+		us = 0;
+	}
 	if (!(c = *str) || ISSPACE(c)) --str;
     }
     c = *str;
@@ -483,7 +501,7 @@ rb_cstr_to_inum(const char *str, int base, int badcheck)
     len *= strlen(str)*sizeof(char);
 
     if (len <= (sizeof(long)*CHAR_BIT)) {
-	unsigned long val = strtoul(str, &end, base);
+	unsigned long val = STRTOUL(str, &end, base);
 
 	if (str < end && *end == '_') goto bigparse;
 	if (badcheck) {
@@ -719,6 +737,7 @@ ceil_log2(register unsigned long x)
 #define MAX_BIG2STR_TABLE_ENTRIES 64
 
 static VALUE big2str_power_cache[35][MAX_BIG2STR_TABLE_ENTRIES];
+static int power_cache_initialized = 0;
 
 static void
 power_cache_init(void)
@@ -810,6 +829,9 @@ big2str_find_n1(VALUE x, int base)
     else if (BIGZEROP(x)) {
         return 0;
     }
+    else if (RBIGNUM_LEN(x) >= LONG_MAX/BITSPERDIG) {
+	rb_raise(rb_eRangeError, "bignum too big to convert into `string'");
+    }
     else {
         bits = BITSPERDIG*RBIGNUM_LEN(x);
     }
@@ -883,6 +905,10 @@ big2str_karatsuba(VALUE x, int base, char* ptr,
         return big2str_orig(x, base, ptr, len, hbase, trim);
     }
 
+    if (!power_cache_initialized) {
+	power_cache_init();
+	power_cache_initialized = 1;
+    }
     b = power_cache_get_power(base, n1, &m1);
     bigdivmod(x, b, &q, &r);
     lh = big2str_karatsuba(q, base, ptr,      (len - m1)/2,
@@ -905,7 +931,7 @@ rb_big2str0(VALUE x, int base, int trim)
         return rb_fix2str(x, base);
     }
     if (BIGZEROP(x)) {
-        return rb_str_new2("0");
+        return rb_usascii_str_new2("0");
     }
 
     if (base < 2 || 36 < base)
@@ -913,7 +939,7 @@ rb_big2str0(VALUE x, int base, int trim)
 
     n2 = big2str_find_n1(x, base);
     n1 = (n2 + 1) / 2;
-    ss = rb_str_new(0, n2 + 1); /* plus one for sign */
+    ss = rb_usascii_str_new(0, n2 + 1); /* plus one for sign */
     ptr = RSTRING_PTR(ss);
     ptr[0] = RBIGNUM_SIGN(x) ? '+' : '-';
 
@@ -1178,7 +1204,7 @@ rb_big_cmp(VALUE x, VALUE y)
 	return rb_dbl_cmp(rb_big2dbl(x), RFLOAT_VALUE(y));
 
       default:
-	return rb_num_coerce_cmp(x, y);
+	return rb_num_coerce_cmp(x, y, rb_intern("<=>"));
     }
 
     if (RBIGNUM_SIGN(x) > RBIGNUM_SIGN(y)) return INT2FIX(1);
@@ -1413,7 +1439,7 @@ rb_big_plus(VALUE x, VALUE y)
 	return DOUBLE2NUM(rb_big2dbl(x) + RFLOAT_VALUE(y));
 
       default:
-	return rb_num_coerce_bin(x, y);
+	return rb_num_coerce_bin(x, y, '+');
     }
 }
 
@@ -1438,7 +1464,7 @@ rb_big_minus(VALUE x, VALUE y)
 	return DOUBLE2NUM(rb_big2dbl(x) - RFLOAT_VALUE(y));
 
       default:
-	return rb_num_coerce_bin(x, y);
+	return rb_num_coerce_bin(x, y, '-');
     }
 }
 
@@ -1450,7 +1476,7 @@ rb_big_stop(void *ptr)
 }
 
 struct big_mul_struct {
-    VALUE x, y, stop;
+    VALUE x, y, z, stop;
 };
 
 static VALUE
@@ -1459,11 +1485,10 @@ bigmul1(void *ptr)
     struct big_mul_struct *bms = (struct big_mul_struct*)ptr;
     long i, j;
     BDIGIT_DBL n = 0;
-    VALUE x = bms->x, y = bms->y, z;
+    VALUE x = bms->x, y = bms->y, z = bms->z;
     BDIGIT *zds;
 
     j = RBIGNUM_LEN(x) + RBIGNUM_LEN(y) + 1;
-    z = bignew(j, RBIGNUM_SIGN(x)==RBIGNUM_SIGN(y));
     zds = BDIGITS(z);
     while (j--) zds[j] = 0;
     for (i = 0; i < RBIGNUM_LEN(x); i++) {
@@ -1489,7 +1514,7 @@ static VALUE
 rb_big_mul0(VALUE x, VALUE y)
 {
     struct big_mul_struct bms;
-    VALUE z;
+    volatile VALUE z;
 
     switch (TYPE(y)) {
       case T_FIXNUM:
@@ -1503,11 +1528,12 @@ rb_big_mul0(VALUE x, VALUE y)
 	return DOUBLE2NUM(rb_big2dbl(x) * RFLOAT_VALUE(y));
 
       default:
-	return rb_num_coerce_bin(x, y);
+	return rb_num_coerce_bin(x, y, '*');
     }
 
     bms.x = x;
     bms.y = y;
+    bms.z = bignew(RBIGNUM_LEN(x) + RBIGNUM_LEN(y) + 1, RBIGNUM_SIGN(x)==RBIGNUM_SIGN(y));
     bms.stop = Qfalse;
 
     if (RBIGNUM_LEN(x) + RBIGNUM_LEN(y) > 10000) {
@@ -1534,20 +1560,64 @@ rb_big_mul(VALUE x, VALUE y)
 }
 
 struct big_div_struct {
-    VALUE x, y, *divp, *modp, stop;
+    long nx, ny;
+    BDIGIT *yds, *zds;
+    VALUE stop;
 };
 
 static VALUE
 bigdivrem1(void *ptr)
 {
     struct big_div_struct *bds = (struct big_div_struct*)ptr;
-    VALUE x = bds->x, y = bds->y, *divp = bds->divp, *modp = bds->modp;
-    long nx = RBIGNUM_LEN(x), ny = RBIGNUM_LEN(y);
+    long nx = bds->nx, ny = bds->ny;
     long i, j;
-    VALUE yy, z;
-    BDIGIT *xds, *yds, *zds, *tds;
+    BDIGIT *yds = bds->yds, *zds = bds->zds;
     BDIGIT_DBL t2;
     BDIGIT_DBL_SIGNED num;
+    BDIGIT q;
+
+    j = nx==ny?nx+1:nx;
+    do {
+	if (bds->stop) return Qnil;
+	if (zds[j] ==  yds[ny-1]) q = BIGRAD-1;
+	else q = (BDIGIT)((BIGUP(zds[j]) + zds[j-1])/yds[ny-1]);
+	if (q) {
+	    i = 0; num = 0; t2 = 0;
+	    do {			/* multiply and subtract */
+		BDIGIT_DBL ee;
+		t2 += (BDIGIT_DBL)yds[i] * q;
+		ee = num - BIGLO(t2);
+		num = (BDIGIT_DBL)zds[j - ny + i] + ee;
+		if (ee) zds[j - ny + i] = BIGLO(num);
+		num = BIGDN(num);
+		t2 = BIGDN(t2);
+	    } while (++i < ny);
+	    num += zds[j - ny + i] - t2;/* borrow from high digit; don't update */
+	    while (num) {		/* "add back" required */
+		i = 0; num = 0; q--;
+		do {
+		    BDIGIT_DBL ee = num + yds[i];
+		    num = (BDIGIT_DBL)zds[j - ny + i] + ee;
+		    if (ee) zds[j - ny + i] = BIGLO(num);
+		    num = BIGDN(num);
+		} while (++i < ny);
+		num--;
+	    }
+	}
+	zds[j] = q;
+    } while (--j >= ny);
+    return Qnil;
+}
+
+static VALUE
+bigdivrem(VALUE x, VALUE y, VALUE *divp, VALUE *modp)
+{
+    struct big_div_struct bds;
+    long nx = RBIGNUM_LEN(x), ny = RBIGNUM_LEN(y);
+    long i, j;
+    volatile VALUE yy, z;
+    BDIGIT *xds, *yds, *zds, *tds;
+    BDIGIT_DBL t2;
     BDIGIT dd, q;
 
     if (BIGZEROP(y)) rb_num_zerodiv();
@@ -1613,36 +1683,18 @@ bigdivrem1(void *ptr)
 	while (j--) zds[j] = xds[j];
     }
 
-    j = nx==ny?nx+1:nx;
-    do {
-	if (bds->stop) return Qnil;
-	if (zds[j] ==  yds[ny-1]) q = BIGRAD-1;
-	else q = (BDIGIT)((BIGUP(zds[j]) + zds[j-1])/yds[ny-1]);
-	if (q) {
-	    i = 0; num = 0; t2 = 0;
-	    do {			/* multiply and subtract */
-		BDIGIT_DBL ee;
-		t2 += (BDIGIT_DBL)yds[i] * q;
-		ee = num - BIGLO(t2);
-		num = (BDIGIT_DBL)zds[j - ny + i] + ee;
-		if (ee) zds[j - ny + i] = BIGLO(num);
-		num = BIGDN(num);
-		t2 = BIGDN(t2);
-	    } while (++i < ny);
-	    num += zds[j - ny + i] - t2;/* borrow from high digit; don't update */
-	    while (num) {		/* "add back" required */
-		i = 0; num = 0; q--;
-		do {
-		    BDIGIT_DBL ee = num + yds[i];
-		    num = (BDIGIT_DBL)zds[j - ny + i] + ee;
-		    if (ee) zds[j - ny + i] = BIGLO(num);
-		    num = BIGDN(num);
-		} while (++i < ny);
-		num--;
-	    }
-	}
-	zds[j] = q;
-    } while (--j >= ny);
+    bds.nx = nx;
+    bds.ny = ny;
+    bds.zds = zds;
+    bds.yds = yds;
+    bds.stop = Qfalse;
+    if (RBIGNUM_LEN(x) > 10000 || RBIGNUM_LEN(y) > 10000) {
+	rb_thread_blocking_region(bigdivrem1, &bds, rb_big_stop, &bds.stop);
+    }
+    else {
+	bigdivrem1(&bds);
+    }
+
     if (divp) {			/* move quotient down in z */
 	*divp = rb_big_clone(z);
 	zds = BDIGITS(*divp);
@@ -1666,26 +1718,6 @@ bigdivrem1(void *ptr)
 	RBIGNUM_SET_LEN(*modp, ny);
 	RBIGNUM_SET_SIGN(*modp, RBIGNUM_SIGN(x));
     }
-    return Qnil;
-}
-
-static VALUE
-bigdivrem(VALUE x, VALUE y, VALUE *divp, VALUE *modp)
-{
-    struct big_div_struct bds;
-    VALUE z;
-
-    bds.x = x;
-    bds.y = y;
-    bds.divp = divp;
-    bds.modp = modp;
-    bds.stop = Qfalse;
-    if (RBIGNUM_LEN(x) > 10000 || RBIGNUM_LEN(y) > 10000) {
-	z = rb_thread_blocking_region(bigdivrem1, &bds, rb_big_stop, &bds.stop);
-    }
-    else {
-	z = bigdivrem1(&bds);
-    }
     return z;
 }
 
@@ -1730,7 +1762,7 @@ rb_big_div(VALUE x, VALUE y)
 	return DOUBLE2NUM(rb_big2dbl(x) / RFLOAT_VALUE(y));
 
       default:
-	return rb_num_coerce_bin(x, y);
+	return rb_num_coerce_bin(x, y, '/');
     }
     bigdivmod(x, y, &z, 0);
 
@@ -1760,7 +1792,7 @@ rb_big_modulo(VALUE x, VALUE y)
 	break;
 
       default:
-	return rb_num_coerce_bin(x, y);
+	return rb_num_coerce_bin(x, y, '%');
     }
     bigdivmod(x, y, 0, &z);
 
@@ -1790,7 +1822,7 @@ rb_big_remainder(VALUE x, VALUE y)
 	break;
 
       default:
-	return rb_num_coerce_bin(x, y);
+	return rb_num_coerce_bin(x, y, rb_intern("remainder"));
     }
     bigdivrem(x, y, 0, &z);
 
@@ -1818,7 +1850,7 @@ rb_big_divmod(VALUE x, VALUE y)
 	break;
 
       default:
-	return rb_num_coerce_bin(x, y);
+	return rb_num_coerce_bin(x, y, rb_intern("divmod"));
     }
     bigdivmod(x, y, &div, &mod);
 
@@ -1852,7 +1884,7 @@ static VALUE big_rshift(VALUE, unsigned long);
 static VALUE big_shift(VALUE x, int n)
 {
     if (n < 0)
-	return big_lshift(x, (unsigned int)n);
+	return big_lshift(x, (unsigned int)-n);
     else if (n > 0)
 	return big_rshift(x, (unsigned int)n);
     return x;
@@ -1919,7 +1951,7 @@ rb_big_quo(VALUE x, VALUE y)
 	break;
 
       default:
-	return rb_num_coerce_bin(x, y);
+	return rb_num_coerce_bin(x, y, rb_intern("quo"));
     }
     return DOUBLE2NUM(dx / dy);
 }
@@ -2026,7 +2058,7 @@ rb_big_pow(VALUE x, VALUE y)
 	break;
 
       default:
-	return rb_num_coerce_bin(x, y);
+	return rb_num_coerce_bin(x, y, rb_intern("**"));
     }
     return DOUBLE2NUM(pow(rb_big2dbl(x), d));
 }
@@ -2581,6 +2613,4 @@ Init_Bignum(void)
     rb_define_method(rb_cBignum, "size", rb_big_size, 0);
     rb_define_method(rb_cBignum, "odd?", rb_big_odd_p, 0);
     rb_define_method(rb_cBignum, "even?", rb_big_even_p, 0);
-
-    power_cache_init();
 }
