@@ -2,17 +2,18 @@
 
   enumerator.c - provides Enumerator class
 
-  $Author: knu $
+  $Author: mame $
 
   Copyright (C) 2001-2003 Akinori MUSHA
 
   $Idaemons: /home/cvs/rb/enumerator/enumerator.c,v 1.1.1.1 2001/07/15 10:12:48 knu Exp $
   $RoughId: enumerator.c,v 1.6 2003/07/27 11:03:24 nobu Exp $
-  $Id: enumerator.c 16583 2008-05-24 20:21:59Z knu $
+  $Id: enumerator.c 15326 2008-01-29 13:30:15Z mame $
 
 ************************************************/
 
-#include "ruby.h"
+#include "ruby/ruby.h"
+#include "debug.h"
 
 /*
  * Document-class: Enumerable::Enumerator
@@ -20,32 +21,43 @@
  * A class which provides a method `each' to be used as an Enumerable
  * object.
  */
-VALUE rb_cEnumerator;
-static VALUE sym_each;
+static VALUE rb_cEnumerator;
+static VALUE sym_each, sym_call;
 
 VALUE rb_eStopIteration;
 
+static VALUE
+proc_call(VALUE proc, VALUE args)
+{
+    if (TYPE(args) != T_ARRAY) {
+	args = rb_ary_new3(1, args);
+    }
+    return rb_proc_call(proc, args);
+}
+
 struct enumerator {
-    VALUE obj;
-    ID    meth;
+    VALUE method;
     VALUE proc;
     VALUE args;
     rb_block_call_func *iter;
+    VALUE fib;
+    VALUE dst;
+    VALUE no_next;
 };
 
-static void enumerator_mark _((void *));
 static void
-enumerator_mark(p)
-    void *p;
+enumerator_mark(void *p)
 {
     struct enumerator *ptr = p;
-    rb_gc_mark(ptr->obj);
+    rb_gc_mark(ptr->method);
+    rb_gc_mark(ptr->proc);
     rb_gc_mark(ptr->args);
+    rb_gc_mark(ptr->fib);
+    rb_gc_mark(ptr->dst);
 }
 
 static struct enumerator *
-enumerator_ptr(obj)
-    VALUE obj;
+enumerator_ptr(VALUE obj)
 {
     struct enumerator *ptr;
 
@@ -61,6 +73,13 @@ enumerator_ptr(obj)
     return ptr;
 }
 
+static VALUE
+enumerator_iter_i(VALUE i, VALUE enum_obj, int argc, VALUE *argv)
+{
+    struct enumerator *e = (struct enumerator *)enum_obj;
+    return rb_yield(proc_call(e->proc, i));
+}
+
 /*
  *  call-seq:
  *    obj.to_enum(method = :each, *args)
@@ -69,7 +88,6 @@ enumerator_ptr(obj)
  *  Returns Enumerable::Enumerator.new(self, method, *args).
  *
  *  e.g.:
- *
  *     str = "xyz"
  *
  *     enum = str.enum_for(:each_byte)
@@ -81,10 +99,7 @@ enumerator_ptr(obj)
  *
  */
 static VALUE
-obj_to_enum(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+obj_to_enum(int argc, VALUE *argv, VALUE obj)
 {
     VALUE meth = sym_each;
 
@@ -96,9 +111,7 @@ obj_to_enum(argc, argv, obj)
 }
 
 static VALUE
-each_slice_i(val, memo)
-    VALUE val;
-    VALUE *memo;
+each_slice_i(VALUE val, VALUE *memo)
 {
     VALUE ary = memo[0];
     VALUE v = Qnil;
@@ -117,10 +130,8 @@ each_slice_i(val, memo)
 /*
  *  call-seq:
  *    e.each_slice(n) {...}
- *    e.each_slice(n)
  *
- *  Iterates the given block for each slice of <n> elements.  If no
- *  block is given, returns an enumerator.
+ *  Iterates the given block for each slice of <n> elements.
  *
  *  e.g.:
  *      (1..10).each_slice(3) {|a| p a}
@@ -132,8 +143,7 @@ each_slice_i(val, memo)
  *
  */
 static VALUE
-enum_each_slice(obj, n)
-    VALUE obj, n;
+enum_each_slice(VALUE obj, VALUE n)
 {
     long size = NUM2LONG(n);
     VALUE args[2], ary;
@@ -152,9 +162,7 @@ enum_each_slice(obj, n)
 }
 
 static VALUE
-each_cons_i(val, memo)
-    VALUE val;
-    VALUE *memo;
+each_cons_i(VALUE val, VALUE *memo)
 {
     VALUE ary = memo[0];
     VALUE v = Qnil;
@@ -173,10 +181,9 @@ each_cons_i(val, memo)
 /*
  *  call-seq:
  *    each_cons(n) {...}
- *    each_cons(n)
  *
  *  Iterates the given block for each array of consecutive <n>
- *  elements.  If no block is given, returns an enumerator.a
+ *  elements.
  *
  *  e.g.:
  *      (1..10).each_cons(3) {|a| p a}
@@ -192,8 +199,7 @@ each_cons_i(val, memo)
  *
  */
 static VALUE
-enum_each_cons(obj, n)
-    VALUE obj, n;
+enum_each_cons(VALUE obj, VALUE n)
 {
     long size = NUM2LONG(n);
     VALUE args[2];
@@ -208,39 +214,37 @@ enum_each_cons(obj, n)
     return Qnil;
 }
 
-static VALUE enumerator_allocate _((VALUE));
 static VALUE
-enumerator_allocate(klass)
-    VALUE klass;
+enumerator_allocate(VALUE klass)
 {
     struct enumerator *ptr;
     return Data_Make_Struct(klass, struct enumerator,
 			    enumerator_mark, -1, ptr);
 }
 
-static VALUE enumerator_each_i _((VALUE, VALUE));
 static VALUE
-enumerator_each_i(v, enum_obj)
-    VALUE v;
-    VALUE enum_obj;
+enumerator_each_i(VALUE v, VALUE enum_obj, int argc, VALUE *argv)
 {
-    return rb_yield(v);
+    return rb_yield_values2(argc, argv);
 }
 
 static VALUE
-enumerator_init(enum_obj, obj, meth, argc, argv)
-    VALUE enum_obj;
-    VALUE obj;
-    VALUE meth;
-    int argc;
-    VALUE *argv;
+enumerator_init(VALUE enum_obj, VALUE obj, VALUE meth, int argc, VALUE *argv)
 {
     struct enumerator *ptr = enumerator_ptr(enum_obj);
 
-    ptr->obj  = obj;
-    ptr->meth = rb_to_id(meth);
-    ptr->iter = enumerator_each_i;
+    ptr->method = rb_obj_method(obj, meth);
+    if (rb_block_given_p()) {
+	ptr->proc = rb_block_proc();
+	ptr->iter = enumerator_iter_i;
+    }
+    else {
+	ptr->iter = enumerator_each_i;
+    }
     if (argc) ptr->args = rb_ary_new4(argc, argv);
+    ptr->fib = 0;
+    ptr->dst = Qnil;
+    ptr->no_next = Qfalse;
 
     return enum_obj;
 }
@@ -253,14 +257,15 @@ enumerator_init(enum_obj, obj, meth, argc, argv)
  *  used as an Enumerable object using the given object's given
  *  method with the given arguments.
  *
- *  Use of this method is not discouraged.  Use Kernel#enum_for()
- *  instead.
+ *  e.g.:
+ *      str = "xyz"
+ *
+ *      enum = Enumerable::Enumerator.new(str, :each_byte)
+ *      a = enum.map {|b| '%02x' % b } #=> ["78", "79", "7a"]
+ *
  */
 static VALUE
-enumerator_initialize(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+enumerator_initialize(int argc, VALUE *argv, VALUE obj)
 {
     VALUE recv, meth = sym_each;
 
@@ -276,29 +281,28 @@ enumerator_initialize(argc, argv, obj)
 
 /* :nodoc: */
 static VALUE
-enumerator_init_copy(obj, orig)
-    VALUE obj;
-    VALUE orig;
+enumerator_init_copy(VALUE obj, VALUE orig)
 {
     struct enumerator *ptr0, *ptr1;
 
     ptr0 = enumerator_ptr(orig);
+    if (ptr0->fib) {
+	/* Fibers cannot be copied */
+	rb_raise(rb_eTypeError, "can't copy execution context");
+    }
     ptr1 = enumerator_ptr(obj);
 
-    ptr1->obj  = ptr0->obj;
-    ptr1->meth = ptr0->meth;
+    ptr1->method = ptr0->method;
+    ptr1->proc = ptr0->proc;
     ptr1->iter = ptr0->iter;
     ptr1->args = ptr0->args;
+    ptr1->fib  = 0;
 
     return obj;
 }
 
 VALUE
-rb_enumeratorize(obj, meth, argc, argv)
-    VALUE obj;
-    VALUE meth;
-    int argc;
-    VALUE *argv;
+rb_enumeratorize(VALUE obj, VALUE meth, int argc, VALUE *argv)
 {
     return enumerator_init(enumerator_allocate(rb_cEnumerator), obj, meth, argc, argv);
 }
@@ -308,12 +312,11 @@ rb_enumeratorize(obj, meth, argc, argv)
  *    enum.each {...}
  *
  *  Iterates the given block using the object and the method specified
- *  in the first place.  If no block is given, returns self.
+ *  in the first place.
  *
  */
 static VALUE
-enumerator_each(obj)
-    VALUE obj;
+enumerator_each(VALUE obj)
 {
     struct enumerator *e;
     int argc = 0;
@@ -325,13 +328,11 @@ enumerator_each(obj)
 	argc = RARRAY_LEN(e->args);
 	argv = RARRAY_PTR(e->args);
     }
-    return rb_block_call(e->obj, e->meth, argc, argv, e->iter, (VALUE)e);
+    return rb_block_call(e->method, SYM2ID(sym_call), argc, argv, e->iter, (VALUE)e);
 }
 
 static VALUE
-enumerator_with_index_i(val, memo)
-    VALUE val;
-    VALUE *memo;
+enumerator_with_index_i(VALUE val, VALUE *memo)
 {
     val = rb_yield_values(2, val, INT2FIX(*memo));
     ++*memo;
@@ -341,15 +342,13 @@ enumerator_with_index_i(val, memo)
 /*
  *  call-seq:
  *    e.with_index {|(*args), idx| ... }
- *    e.with_index
  *
  *  Iterates the given block for each elements with an index, which
- *  start from 0.  If no block is given, returns an enumerator.
+ *  start from 0.
  *
  */
 static VALUE
-enumerator_with_index(obj)
-    VALUE obj;
+enumerator_with_index(VALUE obj)
 {
     struct enumerator *e = enumerator_ptr(obj);
     VALUE memo = 0;
@@ -361,10 +360,36 @@ enumerator_with_index(obj)
 	argc = RARRAY_LEN(e->args);
 	argv = RARRAY_PTR(e->args);
     }
-    return rb_block_call(e->obj, e->meth, argc, argv,
+    return rb_block_call(e->method, SYM2ID(sym_call), argc, argv,
 			 enumerator_with_index_i, (VALUE)&memo);
 }
 
+static VALUE
+next_ii(VALUE i, VALUE obj, int argc, VALUE *argv)
+{
+    rb_fiber_yield(argc, argv);
+    return Qnil;
+}
+
+static VALUE
+next_i(VALUE curr, VALUE obj)
+{
+    struct enumerator *e = enumerator_ptr(obj);
+    VALUE nil = Qnil;
+
+    rb_block_call(obj, rb_intern("each"), 0, 0, next_ii, obj);
+    e->no_next = Qtrue;
+    return rb_fiber_yield(1, &nil);
+}
+
+static void
+next_init(VALUE obj, struct enumerator *e)
+{
+    VALUE curr = rb_fiber_current();
+    e->dst = curr;
+    e->fib = rb_fiber_new(next_i, obj);
+}
+
 /*
  * call-seq:
  *   e.next   => object
@@ -377,16 +402,27 @@ enumerator_with_index(obj)
  * non-external enumeration methods, unless underlying iteration
  * methods itself has side-effect, e.g. IO#each_line.
  *
- * Caution: Calling this method causes the "generator" library to be
- * loaded.
  */
 
 static VALUE
-enumerator_next(obj)
-    VALUE obj;
+enumerator_next(VALUE obj)
 {
-    rb_require("generator");
-    return rb_funcall(obj, rb_intern("next"), 0, 0);
+    struct enumerator *e = enumerator_ptr(obj);
+    VALUE curr, v;
+    curr = rb_fiber_current();
+
+    if (!e->fib || !rb_fiber_alive_p(e->fib)) {
+	next_init(obj, e);
+    }
+
+    v = rb_fiber_resume(e->fib, 1, &curr);
+    if (e->no_next) {
+	e->fib = 0;
+	e->dst = Qnil;
+	e->no_next = Qfalse;
+	rb_raise(rb_eStopIteration, "iteration reached at end");
+    }
+    return v;
 }
 
 /*
@@ -397,23 +433,24 @@ enumerator_next(obj)
  */
 
 static VALUE
-enumerator_rewind(obj)
-    VALUE obj;
+enumerator_rewind(VALUE obj)
 {
-    rb_require("generator");
-    return rb_funcall(obj, rb_intern("rewind"), 0, 0);
+    struct enumerator *e = enumerator_ptr(obj);
+
+    e->fib = 0;
+    e->dst = Qnil;
+    e->no_next = Qfalse;
+    return obj;
 }
 
 void
-Init_Enumerator()
+Init_Enumerator(void)
 {
     rb_define_method(rb_mKernel, "to_enum", obj_to_enum, -1);
     rb_define_method(rb_mKernel, "enum_for", obj_to_enum, -1);
 
     rb_define_method(rb_mEnumerable, "each_slice", enum_each_slice, 1);
-    rb_define_method(rb_mEnumerable, "enum_slice", enum_each_slice, 1);
     rb_define_method(rb_mEnumerable, "each_cons", enum_each_cons, 1);
-    rb_define_method(rb_mEnumerable, "enum_cons", enum_each_cons, 1);
 
     rb_cEnumerator = rb_define_class_under(rb_mEnumerable, "Enumerator", rb_cObject);
     rb_include_module(rb_cEnumerator, rb_mEnumerable);
@@ -422,14 +459,14 @@ Init_Enumerator()
     rb_define_method(rb_cEnumerator, "initialize", enumerator_initialize, -1);
     rb_define_method(rb_cEnumerator, "initialize_copy", enumerator_init_copy, 1);
     rb_define_method(rb_cEnumerator, "each", enumerator_each, 0);
-    rb_define_method(rb_cEnumerator, "each_with_index", enumerator_with_index, 0);
     rb_define_method(rb_cEnumerator, "with_index", enumerator_with_index, 0);
     rb_define_method(rb_cEnumerator, "next", enumerator_next, 0);
     rb_define_method(rb_cEnumerator, "rewind", enumerator_rewind, 0);
 
     rb_eStopIteration   = rb_define_class("StopIteration", rb_eIndexError);
 
-    sym_each	 	= ID2SYM(rb_intern("each"));
+    sym_each		= ID2SYM(rb_intern("each"));
+    sym_call		= ID2SYM(rb_intern("call"));
 
     rb_provide("enumerator.so");	/* for backward compatibility */
 }
