@@ -1,5 +1,5 @@
 /* -*- C -*-
- * $Id: curses.c 25899 2009-11-24 07:02:04Z shyouhei $
+ * $Id: curses.c 12095 2007-03-19 03:58:57Z matz $
  *
  * ext/curses/curses.c
  * 
@@ -54,8 +54,8 @@
 # define USE_MOUSE 1
 #endif
 
-#define NUM2CH NUM2LONG
-#define CH2FIX LONG2FIX
+#define NUM2CH NUM2CHR
+#define CH2FIX CHR2FIX
 
 static VALUE mCurses;
 static VALUE mKey;
@@ -72,12 +72,12 @@ struct windata {
 
 #define CHECK(c) c
 
-static VALUE window_attroff();
-static VALUE window_attron();
-static VALUE window_attrset();
+static VALUE window_attroff(VALUE obj, VALUE attrs);
+static VALUE window_attron(VALUE obj, VALUE attrs);
+static VALUE window_attrset(VALUE obj, VALUE attrs);
 
 static void
-no_window()
+no_window(void)
 {
     rb_raise(rb_eRuntimeError, "already closed window");
 }
@@ -90,8 +90,7 @@ no_window()
 } while (0)
 
 static void
-free_window(winp)
-    struct windata *winp;
+free_window(struct windata *winp)
 {
     if (winp->window && winp->window != stdscr) delwin(winp->window);
     winp->window = 0;
@@ -99,9 +98,7 @@ free_window(winp)
 }
 
 static VALUE
-prep_window(class, window)
-    VALUE class;
-    WINDOW *window;
+prep_window(VALUE class, WINDOW *window)
 {
     VALUE obj;
     struct windata *winp;
@@ -121,7 +118,7 @@ prep_window(class, window)
 
 /* def init_screen */
 static VALUE
-curses_init_screen()
+curses_init_screen(void)
 {
     rb_secure(4);
     if (rb_stdscr) return rb_stdscr;
@@ -139,9 +136,8 @@ curses_init_screen()
 
 /* def close_screen */
 static VALUE
-curses_close_screen()
+curses_close_screen(void)
 {
-    curses_stdscr();
 #ifdef HAVE_ISENDWIN
     if (!isendwin())
 #endif
@@ -165,10 +161,9 @@ curses_finalize(VALUE dummy)
 
 /* def closed? */
 static VALUE
-curses_closed()
+curses_closed(void)
 {
 #ifdef HAVE_ISENDWIN
-    curses_stdscr();
     if (isendwin()) {
 	return Qtrue;
     }
@@ -180,8 +175,7 @@ curses_closed()
 
 /* def clear */
 static VALUE
-curses_clear(obj)
-    VALUE obj;
+curses_clear(VALUE obj)
 {
     curses_stdscr();
     wclear(stdscr);
@@ -190,7 +184,7 @@ curses_clear(obj)
 
 /* def clrtoeol */
 static VALUE
-curses_clrtoeol()
+curses_clrtoeol(void)
 {
     curses_stdscr();
     clrtoeol();
@@ -199,8 +193,7 @@ curses_clrtoeol()
 
 /* def refresh */
 static VALUE
-curses_refresh(obj)
-    VALUE obj;
+curses_refresh(VALUE obj)
 {
     curses_stdscr();
     refresh();
@@ -209,8 +202,7 @@ curses_refresh(obj)
 
 /* def doupdate */
 static VALUE
-curses_doupdate(obj)
-    VALUE obj;
+curses_doupdate(VALUE obj)
 {
     curses_stdscr();
 #ifdef HAVE_DOUPDATE
@@ -223,8 +215,7 @@ curses_doupdate(obj)
 
 /* def echo */
 static VALUE
-curses_echo(obj)
-    VALUE obj;
+curses_echo(VALUE obj)
 {
     curses_stdscr();
     echo();
@@ -233,8 +224,7 @@ curses_echo(obj)
 
 /* def noecho */
 static VALUE
-curses_noecho(obj)
-    VALUE obj;
+curses_noecho(VALUE obj)
 {
     curses_stdscr();
     noecho();
@@ -243,8 +233,7 @@ curses_noecho(obj)
 
 /* def raw */
 static VALUE
-curses_raw(obj)
-    VALUE obj;
+curses_raw(VALUE obj)
 {
     curses_stdscr();
     raw();
@@ -253,8 +242,7 @@ curses_raw(obj)
 
 /* def noraw */
 static VALUE
-curses_noraw(obj)
-    VALUE obj;
+curses_noraw(VALUE obj)
 {
     curses_stdscr();
     noraw();
@@ -263,8 +251,7 @@ curses_noraw(obj)
 
 /* def cbreak */
 static VALUE
-curses_cbreak(obj)
-    VALUE obj;
+curses_cbreak(VALUE obj)
 {
     curses_stdscr();
     cbreak();
@@ -273,8 +260,7 @@ curses_cbreak(obj)
 
 /* def nocbreak */
 static VALUE
-curses_nocbreak(obj)
-    VALUE obj;
+curses_nocbreak(VALUE obj)
 {
     curses_stdscr();
     nocbreak();
@@ -283,8 +269,7 @@ curses_nocbreak(obj)
 
 /* def nl */
 static VALUE
-curses_nl(obj)
-    VALUE obj;
+curses_nl(VALUE obj)
 {
     curses_stdscr();
     nl();
@@ -293,8 +278,7 @@ curses_nl(obj)
 
 /* def nonl */
 static VALUE
-curses_nonl(obj)
-    VALUE obj;
+curses_nonl(VALUE obj)
 {
     curses_stdscr();
     nonl();
@@ -303,8 +287,7 @@ curses_nonl(obj)
 
 /* def beep */
 static VALUE
-curses_beep(obj)
-    VALUE obj;
+curses_beep(VALUE obj)
 {
 #ifdef HAVE_BEEP
     curses_stdscr();
@@ -315,8 +298,7 @@ curses_beep(obj)
 
 /* def flash */
 static VALUE
-curses_flash(obj)
-    VALUE obj;
+curses_flash(VALUE obj)
 {
 #ifdef HAVE_FLASH
     curses_stdscr();
@@ -327,9 +309,7 @@ curses_flash(obj)
 
 /* def ungetch */
 static VALUE
-curses_ungetch(obj, ch)
-    VALUE obj;
-    VALUE ch;
+curses_ungetch(VALUE obj, VALUE ch)
 {
 #ifdef HAVE_UNGETCH
     curses_stdscr();
@@ -342,10 +322,7 @@ curses_ungetch(obj, ch)
 
 /* def setpos(y, x) */
 static VALUE
-curses_setpos(obj, y, x)
-    VALUE obj;
-    VALUE y;
-    VALUE x;
+curses_setpos(VALUE obj, VALUE y, VALUE x)
 {
     curses_stdscr();
     move(NUM2INT(y), NUM2INT(x));
@@ -354,28 +331,23 @@ curses_setpos(obj, y, x)
 
 /* def standout */
 static VALUE
-curses_standout(obj)
-    VALUE obj;
+curses_standout(VALUE obj)
 {
-    curses_stdscr();
     standout();
     return Qnil;
 }
 
 /* def standend */
 static VALUE
-curses_standend(obj)
-    VALUE obj;
+curses_standend(VALUE obj)
 {
-    curses_stdscr();
     standend();
     return Qnil;
 }
 
 /* def inch */
 static VALUE
-curses_inch(obj)
-    VALUE obj;
+curses_inch(VALUE obj)
 {
     curses_stdscr();
     return CH2FIX(inch());
@@ -383,9 +355,7 @@ curses_inch(obj)
 
 /* def addch(ch) */
 static VALUE
-curses_addch(obj, ch)
-    VALUE obj;
-    VALUE ch;
+curses_addch(VALUE obj, VALUE ch)
 {
     curses_stdscr();
     addch(NUM2CH(ch));
@@ -394,9 +364,7 @@ curses_addch(obj, ch)
 
 /* def insch(ch) */
 static VALUE
-curses_insch(obj, ch)
-    VALUE obj;
-    VALUE ch;
+curses_insch(VALUE obj, VALUE ch)
 {
     curses_stdscr();
     insch(NUM2CH(ch));
@@ -405,9 +373,7 @@ curses_insch(obj, ch)
 
 /* def addstr(str) */
 static VALUE
-curses_addstr(obj, str)
-    VALUE obj;
-    VALUE str;
+curses_addstr(VALUE obj, VALUE str)
 {
     curses_stdscr();
     if (!NIL_P(str)) {
@@ -418,8 +384,7 @@ curses_addstr(obj, str)
 
 /* def getch */
 static VALUE
-curses_getch(obj)
-    VALUE obj;
+curses_getch(VALUE obj)
 {
     rb_read_check(stdin);
     curses_stdscr();
@@ -428,12 +393,10 @@ curses_getch(obj)
 
 /* def getstr */
 static VALUE
-curses_getstr(obj)
-    VALUE obj;
+curses_getstr(VALUE obj)
 {
     char rtn[1024]; /* This should be big enough.. I hope */
 
-    curses_stdscr();
     rb_read_check(stdin);
 #if defined(HAVE_GETNSTR)
     getnstr(rtn,1023);
@@ -445,20 +408,16 @@ curses_getstr(obj)
 
 /* def delch */
 static VALUE
-curses_delch(obj)
-    VALUE obj;
+curses_delch(VALUE obj)
 {
-    curses_stdscr();
     delch();
     return Qnil;
 }
 
 /* def delelteln */
 static VALUE
-curses_deleteln(obj)
-    VALUE obj;
+curses_deleteln(VALUE obj)
 {
-    curses_stdscr();
 #if defined(HAVE_DELETELN) || defined(deleteln)
     deleteln();
 #endif
@@ -467,10 +426,8 @@ curses_deleteln(obj)
 
 /* def insertln */
 static VALUE
-curses_insertln(obj)
-    VALUE obj;
+curses_insertln(VALUE obj)
 {
-    curses_stdscr();
 #if defined(HAVE_INSERTLN) || defined(insertln)
     insertln();
 #endif
@@ -479,14 +436,11 @@ curses_insertln(obj)
 
 /* def keyname */
 static VALUE
-curses_keyname(obj, c)
-    VALUE obj;
-    VALUE c;
+curses_keyname(VALUE obj, VALUE c)
 {
 #ifdef HAVE_KEYNAME
   const char *name;
 
-  curses_stdscr();
   name = keyname(NUM2INT(c));
   if (name) {
     return rb_str_new2(name);
@@ -499,13 +453,13 @@ curses_keyname(obj, c)
 }
 
 static VALUE
-curses_lines()
+curses_lines(void)
 {
     return INT2FIX(LINES);
 }
 
 static VALUE
-curses_cols()
+curses_cols(void)
 {
     return INT2FIX(COLS);
 }
@@ -515,7 +469,6 @@ curses_curs_set(VALUE obj, VALUE visibility)
 {
 #ifdef HAVE_CURS_SET
   int n;
-  curses_stdscr();
   return (n = curs_set(NUM2INT(visibility)) != ERR) ? INT2FIX(n) : Qnil;
 #else
   return Qnil;
@@ -527,7 +480,6 @@ curses_scrl(VALUE obj, VALUE n)
 {
   /* may have to raise exception on ERR */
 #ifdef HAVE_SCRL
-  curses_stdscr();
   return (scrl(NUM2INT(n)) == OK) ? Qtrue : Qfalse;
 #else
   return Qfalse;
@@ -539,7 +491,6 @@ curses_setscrreg(VALUE obj, VALUE top, VALUE bottom)
 {
   /* may have to raise exception on ERR */
 #ifdef HAVE_SETSCRREG
-  curses_stdscr();
   return (setscrreg(NUM2INT(top), NUM2INT(bottom)) == OK) ? Qtrue : Qfalse;
 #else
   return Qfalse;
@@ -549,7 +500,6 @@ curses_setscrreg(VALUE obj, VALUE top, VALUE bottom)
 static VALUE
 curses_attroff(VALUE obj, VALUE attrs)
 {
-  curses_stdscr();
   return window_attroff(rb_stdscr,attrs);  
   /* return INT2FIX(attroff(NUM2INT(attrs))); */
 }
@@ -557,7 +507,6 @@ curses_attroff(VALUE obj, VALUE attrs)
 static VALUE
 curses_attron(VALUE obj, VALUE attrs)
 {
-  curses_stdscr();
   return window_attron(rb_stdscr,attrs);
   /* return INT2FIX(attroff(NUM2INT(attrs))); */
 }
@@ -565,7 +514,6 @@ curses_attron(VALUE obj, VALUE attrs)
 static VALUE
 curses_attrset(VALUE obj, VALUE attrs)
 {
-  curses_stdscr();
   return window_attrset(rb_stdscr,attrs);
   /* return INT2FIX(attroff(NUM2INT(attrs))); */
 }
@@ -574,7 +522,6 @@ static VALUE
 curses_bkgdset(VALUE obj, VALUE ch)
 {
 #ifdef HAVE_BKGDSET
-  curses_stdscr();
   bkgdset(NUM2CH(ch));
 #endif
   return Qnil;
@@ -584,7 +531,6 @@ static VALUE
 curses_bkgd(VALUE obj, VALUE ch)
 {
 #ifdef HAVE_BKGD
-  curses_stdscr();
   return (bkgd(NUM2CH(ch)) == OK) ? Qtrue : Qfalse;
 #else
   return Qfalse;
@@ -595,7 +541,6 @@ static VALUE
 curses_resizeterm(VALUE obj, VALUE lin, VALUE col)
 {
 #if defined(HAVE_RESIZETERM)
-  curses_stdscr();
   return (resizeterm(NUM2INT(lin),NUM2INT(col)) == OK) ? Qtrue : Qfalse;
 #else
   return Qnil;
@@ -607,7 +552,6 @@ static VALUE
 curses_start_color(VALUE obj)
 {
   /* may have to raise exception on ERR */
-  curses_stdscr();
   return (start_color() == OK) ? Qtrue : Qfalse;
 }
 
@@ -615,7 +559,6 @@ static VALUE
 curses_init_pair(VALUE obj, VALUE pair, VALUE f, VALUE b)
 {
   /* may have to raise exception on ERR */
-  curses_stdscr();
   return (init_pair(NUM2INT(pair),NUM2INT(f),NUM2INT(b)) == OK) ? Qtrue : Qfalse;
 }
 
@@ -623,7 +566,6 @@ static VALUE
 curses_init_color(VALUE obj, VALUE color, VALUE r, VALUE g, VALUE b)
 {
   /* may have to raise exception on ERR */
-  curses_stdscr();
   return (init_color(NUM2INT(color),NUM2INT(r),
 		     NUM2INT(g),NUM2INT(b)) == OK) ? Qtrue : Qfalse;
 }
@@ -631,14 +573,12 @@ curses_init_color(VALUE obj, VALUE color, VALUE r, VALUE g, VALUE b)
 static VALUE
 curses_has_colors(VALUE obj)
 {
-  curses_stdscr();
   return has_colors() ? Qtrue : Qfalse;
 }
 
 static VALUE
 curses_can_change_color(VALUE obj)
 {
-  curses_stdscr();
   return can_change_color() ? Qtrue : Qfalse;
 }
 
@@ -647,7 +587,6 @@ curses_color_content(VALUE obj, VALUE color)
 {
   short r,g,b;
 
-  curses_stdscr();
   color_content(NUM2INT(color),&r,&g,&b);
   return rb_ary_new3(3,INT2FIX(r),INT2FIX(g),INT2FIX(b));
 }
@@ -657,7 +596,6 @@ curses_pair_content(VALUE obj, VALUE pair)
 {
   short f,b;
 
-  curses_stdscr();
   pair_content(NUM2INT(pair),&f,&b);
   return rb_ary_new3(2,INT2FIX(f),INT2FIX(b));
 }
@@ -671,7 +609,6 @@ curses_color_pair(VALUE obj, VALUE attrs)
 static VALUE
 curses_pair_number(VALUE obj, VALUE attrs)
 {
-  curses_stdscr();
   return INT2FIX(PAIR_NUMBER(NUM2INT(attrs)));
 }
 #endif /* USE_COLOR */
@@ -682,7 +619,7 @@ struct mousedata {
 };
 
 static void
-no_mevent()
+no_mevent(void)
 {
   rb_raise(rb_eRuntimeError, "no such mouse event");
 }
@@ -707,7 +644,6 @@ curses_getmouse(VALUE obj)
   struct mousedata *mdata;
   VALUE val;
 
-  curses_stdscr();
   val = Data_Make_Struct(cMouseEvent,struct mousedata,
 			 0,curses_mousedata_free,mdata);
   mdata->mevent = (MEVENT*)xmalloc(sizeof(MEVENT));
@@ -719,7 +655,6 @@ curses_ungetmouse(VALUE obj, VALUE mevent)
 {
   struct mousedata *mdata;
 
-  curses_stdscr();
   GetMOUSE(mevent,mdata);
   return (ungetmouse(mdata->mevent) == OK) ? Qtrue : Qfalse;
 }
@@ -727,14 +662,12 @@ curses_ungetmouse(VALUE obj, VALUE mevent)
 static VALUE
 curses_mouseinterval(VALUE obj, VALUE interval)
 {
-  curses_stdscr();
   return mouseinterval(NUM2INT(interval)) ? Qtrue : Qfalse;
 }
 
 static VALUE
 curses_mousemask(VALUE obj, VALUE mask)
 {
-  curses_stdscr();
   return INT2NUM(mousemask(NUM2UINT(mask),NULL));
 }
 
@@ -758,7 +691,6 @@ static VALUE
 curses_timeout(VALUE obj, VALUE delay)
 {
 #ifdef HAVE_TIMEOUT
-  curses_stdscr();
   timeout(NUM2INT(delay));
   return Qnil;
 #else
@@ -770,7 +702,6 @@ static VALUE
 curses_def_prog_mode(VALUE obj)
 {
 #ifdef HAVE_DEF_PROG_MODE
-  curses_stdscr();
   return def_prog_mode() == OK ? Qtrue : Qfalse;
 #else
     rb_notimplement();
@@ -781,7 +712,6 @@ static VALUE
 curses_reset_prog_mode(VALUE obj)
 {
 #ifdef HAVE_RESET_PROG_MODE
-  curses_stdscr();
   return reset_prog_mode() == OK ? Qtrue : Qfalse;
 #else
     rb_notimplement();
@@ -801,12 +731,7 @@ window_s_allocate(VALUE class)
 
 /* def initialize(h, w, top, left) */
 static VALUE
-window_initialize(obj, h, w, top, left)
-    VALUE obj;
-    VALUE h;
-    VALUE w;
-    VALUE top;
-    VALUE left;
+window_initialize(VALUE obj, VALUE h, VALUE w, VALUE top, VALUE left)
 {
     struct windata *winp;
     WINDOW *window;
@@ -824,12 +749,7 @@ window_initialize(obj, h, w, top, left)
 
 /* def subwin(height, width, top, left) */
 static VALUE
-window_subwin(obj, height, width, top, left)
-    VALUE obj;
-    VALUE height;
-    VALUE width;
-    VALUE top;
-    VALUE left;
+window_subwin(VALUE obj, VALUE height, VALUE width, VALUE top, VALUE left)
 {
     struct windata *winp;
     WINDOW *window;
@@ -849,8 +769,7 @@ window_subwin(obj, height, width, top, left)
 
 /* def close */
 static VALUE
-window_close(obj)
-    VALUE obj;
+window_close(VALUE obj)
 {
     struct windata *winp;
     
@@ -863,8 +782,7 @@ window_close(obj)
 
 /* def clear */
 static VALUE
-window_clear(obj)
-    VALUE obj;
+window_clear(VALUE obj)
 {
     struct windata *winp;
     
@@ -876,8 +794,7 @@ window_clear(obj)
 
 /* def clrtoeol */
 static VALUE
-window_clrtoeol(obj)
-    VALUE obj;
+window_clrtoeol(VALUE obj)
 {
     struct windata *winp;
     
@@ -889,8 +806,7 @@ window_clrtoeol(obj)
 
 /* def refresh */
 static VALUE
-window_refresh(obj)
-    VALUE obj;
+window_refresh(VALUE obj)
 {
     struct windata *winp;
     
@@ -902,8 +818,7 @@ window_refresh(obj)
 
 /* def noutrefresh */
 static VALUE
-window_noutrefresh(obj)
-    VALUE obj;
+window_noutrefresh(VALUE obj)
 {
     struct windata *winp;
 
@@ -919,10 +834,7 @@ window_noutrefresh(obj)
 
 /* def move(y, x) */
 static VALUE
-window_move(obj, y, x)
-    VALUE obj;
-    VALUE y;
-    VALUE x;
+window_move(VALUE obj, VALUE y, VALUE x)
 {
     struct windata *winp;
     
@@ -934,10 +846,7 @@ window_move(obj, y, x)
 
 /* def setpos(y, x) */
 static VALUE
-window_setpos(obj, y, x)
-    VALUE obj;
-    VALUE y;
-    VALUE x;
+window_setpos(VALUE obj, VALUE y, VALUE x)
 {
     struct windata *winp;
     
@@ -948,8 +857,7 @@ window_setpos(obj, y, x)
 
 /* def cury */
 static VALUE
-window_cury(obj)
-    VALUE obj;
+window_cury(VALUE obj)
 {
     struct windata *winp;
     int x, y;
@@ -961,8 +869,7 @@ window_cury(obj)
 
 /* def curx */
 static VALUE
-window_curx(obj)
-    VALUE obj;
+window_curx(VALUE obj)
 {
     struct windata *winp;
     int x, y;
@@ -974,8 +881,7 @@ window_curx(obj)
 
 /* def maxy */
 static VALUE
-window_maxy(obj)
-    VALUE obj;
+window_maxy(VALUE obj)
 {
     struct windata *winp;
 
@@ -995,8 +901,7 @@ window_maxy(obj)
 
 /* def maxx */
 static VALUE
-window_maxx(obj)
-    VALUE obj;
+window_maxx(VALUE obj)
 {
     struct windata *winp;
 
@@ -1016,8 +921,7 @@ window_maxx(obj)
 
 /* def begy */
 static VALUE
-window_begy(obj)
-    VALUE obj;
+window_begy(VALUE obj)
 {
     struct windata *winp;
     int x, y;
@@ -1033,8 +937,7 @@ window_begy(obj)
 
 /* def begx */
 static VALUE
-window_begx(obj)
-    VALUE obj;
+window_begx(VALUE obj)
 {
     struct windata *winp;
     int x, y;
@@ -1050,9 +953,7 @@ window_begx(obj)
 
 /* def box(vert, hor) */
 static VALUE
-window_box(argc, argv, self)
-    int argc;
-    VALUE argv[], self;
+window_box(int argc, VALUE *argv, VALUE self)
 {
     struct windata *winp; 
     VALUE vert, hor, corn;
@@ -1086,8 +987,7 @@ window_box(argc, argv, self)
 
 /* def standout */
 static VALUE
-window_standout(obj)
-    VALUE obj;
+window_standout(VALUE obj)
 {
     struct windata *winp;
     
@@ -1098,8 +998,7 @@ window_standout(obj)
 
 /* def standend */
 static VALUE
-window_standend(obj)
-    VALUE obj;
+window_standend(VALUE obj)
 {
     struct windata *winp;
     
@@ -1110,8 +1009,7 @@ window_standend(obj)
 
 /* def inch */
 static VALUE
-window_inch(obj)
-    VALUE obj;
+window_inch(VALUE obj)
 {
     struct windata *winp;
     
@@ -1121,9 +1019,7 @@ window_inch(obj)
 
 /* def addch(ch) */
 static VALUE
-window_addch(obj, ch)
-    VALUE obj;
-    VALUE ch;
+window_addch(VALUE obj, VALUE ch)
 {
     struct windata *winp;
     
@@ -1135,9 +1031,7 @@ window_addch(obj, ch)
 
 /* def insch(ch) */
 static VALUE
-window_insch(obj, ch)
-    VALUE obj;
-    VALUE ch;
+window_insch(VALUE obj, VALUE ch)
 {
     struct windata *winp;
     
@@ -1149,9 +1043,7 @@ window_insch(obj, ch)
 
 /* def addstr(str) */
 static VALUE
-window_addstr(obj, str)
-    VALUE obj;
-    VALUE str;
+window_addstr(VALUE obj, VALUE str)
 {
     if (!NIL_P(str)) {
 	struct windata *winp;
@@ -1164,9 +1056,7 @@ window_addstr(obj, str)
 
 /* def <<(str) */
 static VALUE
-window_addstr2(obj, str)
-    VALUE obj;
-    VALUE str;
+window_addstr2(VALUE obj, VALUE str)
 {
     window_addstr(obj, str);
     return obj;
@@ -1174,8 +1064,7 @@ window_addstr2(obj, str)
 
 /* def getch */
 static VALUE
-window_getch(obj)
-    VALUE obj;
+window_getch(VALUE obj)
 {
     struct windata *winp;
     
@@ -1186,8 +1075,7 @@ window_getch(obj)
 
 /* def getstr */
 static VALUE
-window_getstr(obj)
-    VALUE obj;
+window_getstr(VALUE obj)
 {
     struct windata *winp;
     char rtn[1024]; /* This should be big enough.. I hope */
@@ -1204,8 +1092,7 @@ window_getstr(obj)
 
 /* def delch */
 static VALUE
-window_delch(obj)
-    VALUE obj;
+window_delch(VALUE obj)
 {
     struct windata *winp;
     
@@ -1216,8 +1103,7 @@ window_delch(obj)
 
 /* def delelteln */
 static VALUE
-window_deleteln(obj)
-    VALUE obj;
+window_deleteln(VALUE obj)
 {
 #if defined(HAVE_WDELETELN) || defined(wdeleteln)
     struct windata *winp;
@@ -1230,8 +1116,7 @@ window_deleteln(obj)
 
 /* def insertln */
 static VALUE
-window_insertln(obj)
-    VALUE obj;
+window_insertln(VALUE obj)
 {
 #if defined(HAVE_WINSERTLN) || defined(winsertln)
     struct windata *winp;
@@ -1289,7 +1174,7 @@ window_color_set(VALUE obj, VALUE col)
   res = wcolor_set(winp->window, NUM2INT(col), NULL);
   return (res == OK) ? Qtrue : Qfalse;
 }
-#endif /* USE_COLOR */
+#endif /* defined(USE_COLOR) && defined(HAVE_WCOLOR_SET) */
 
 static VALUE
 window_scroll(VALUE obj)
@@ -1472,7 +1357,7 @@ window_timeout(VALUE obj, VALUE delay)
 
 /*------------------------- Initialization -------------------------*/
 void
-Init_curses()
+Init_curses(void)
 {
     mCurses    = rb_define_module("Curses");
     mKey       = rb_define_module_under(mCurses, "Key");
