@@ -2,8 +2,7 @@
 
   sdbminit.c -
 
-  $Author: ocean $
-  $Date: 2005/06/25 05:42:41 $
+  $Author: akr $
   created at: Fri May  7 08:34:24 JST 1999
 
   Copyright (C) 1995-2001 Yukihiro Matsumoto
@@ -41,8 +40,7 @@ closed_sdbm()
 }
 
 static void
-free_sdbm(dbmp)
-    struct dbmdata *dbmp;
+free_sdbm(struct dbmdata *dbmp)
 {
 
     if (dbmp->di_dbm) sdbm_close(dbmp->di_dbm);
@@ -50,8 +48,7 @@ free_sdbm(dbmp)
 }
 
 static VALUE
-fsdbm_close(obj)
-    VALUE obj;
+fsdbm_close(VALUE obj)
 {
     struct dbmdata *dbmp;
 
@@ -63,8 +60,7 @@ fsdbm_close(obj)
 }
 
 static VALUE
-fsdbm_closed(obj)
-    VALUE obj;
+fsdbm_closed(VALUE obj)
 {
     struct dbmdata *dbmp;
 
@@ -77,19 +73,14 @@ fsdbm_closed(obj)
     return Qfalse;
 }
 
-static VALUE fsdbm_alloc _((VALUE));
 static VALUE
-fsdbm_alloc(klass)
-    VALUE klass;
+fsdbm_alloc(VALUE klass)
 {
     return Data_Wrap_Struct(klass, 0, free_sdbm, 0);
 }
 
 static VALUE
-fsdbm_initialize(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+fsdbm_initialize(int argc, VALUE *argv, VALUE obj)
 {
     VALUE file, vmode;
     DBM *dbm;
@@ -109,15 +100,15 @@ fsdbm_initialize(argc, argv, obj)
 
     dbm = 0;
     if (mode >= 0)
-	dbm = sdbm_open(RSTRING(file)->ptr, O_RDWR|O_CREAT, mode);
+	dbm = sdbm_open(RSTRING_PTR(file), O_RDWR|O_CREAT, mode);
     if (!dbm)
-	dbm = sdbm_open(RSTRING(file)->ptr, O_RDWR, 0);
+	dbm = sdbm_open(RSTRING_PTR(file), O_RDWR, 0);
     if (!dbm)
-	dbm = sdbm_open(RSTRING(file)->ptr, O_RDONLY, 0);
+	dbm = sdbm_open(RSTRING_PTR(file), O_RDONLY, 0);
 
     if (!dbm) {
 	if (mode == -1) return Qnil;
-	rb_sys_fail(RSTRING(file)->ptr);
+	rb_sys_fail(RSTRING_PTR(file));
     }
 
     dbmp = ALLOC(struct dbmdata);
@@ -129,10 +120,7 @@ fsdbm_initialize(argc, argv, obj)
 }
 
 static VALUE
-fsdbm_s_open(argc, argv, klass)
-    int argc;
-    VALUE *argv;
-    VALUE klass;
+fsdbm_s_open(int argc, VALUE *argv, VALUE klass)
 {
     VALUE obj = Data_Wrap_Struct(klass, 0, free_sdbm, 0);
 
@@ -148,16 +136,15 @@ fsdbm_s_open(argc, argv, klass)
 }
 
 static VALUE
-fsdbm_fetch(obj, keystr, ifnone)
-    VALUE obj, keystr, ifnone;
+fsdbm_fetch(VALUE obj, VALUE keystr, VALUE ifnone)
 {
     datum key, value;
     struct dbmdata *dbmp;
     DBM *dbm;
 
     StringValue(keystr);
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
+    key.dptr = RSTRING_PTR(keystr);
+    key.dsize = RSTRING_LEN(keystr);
 
     GetDBM2(obj, dbmp, dbm);
     value = sdbm_fetch(dbm, key);
@@ -170,17 +157,13 @@ fsdbm_fetch(obj, keystr, ifnone)
 }
 
 static VALUE
-fsdbm_aref(obj, keystr)
-    VALUE obj, keystr;
+fsdbm_aref(VALUE obj, VALUE keystr)
 {
     return fsdbm_fetch(obj, keystr, Qnil);
 }
 
 static VALUE
-fsdbm_fetch_m(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+fsdbm_fetch_m(int argc, VALUE *argv, VALUE obj)
 {
     VALUE keystr, valstr, ifnone;
 
@@ -193,90 +176,52 @@ fsdbm_fetch_m(argc, argv, obj)
 }
 
 static VALUE
-fsdbm_index(obj, valstr)
-    VALUE obj, valstr;
+fsdbm_index(VALUE obj, VALUE valstr)
 {
     datum key, val;
     struct dbmdata *dbmp;
     DBM *dbm;
 
     StringValue(valstr);
-    val.dptr = RSTRING(valstr)->ptr;
-    val.dsize = RSTRING(valstr)->len;
+    val.dptr = RSTRING_PTR(valstr);
+    val.dsize = RSTRING_LEN(valstr);
 
     GetDBM2(obj, dbmp, dbm);
     for (key = sdbm_firstkey(dbm); key.dptr; key = sdbm_nextkey(dbm)) {
 	val = sdbm_fetch(dbm, key);
-	if (val.dsize == RSTRING(valstr)->len &&
-	    memcmp(val.dptr, RSTRING(valstr)->ptr, val.dsize) == 0)
+	if (val.dsize == RSTRING_LEN(valstr) &&
+	    memcmp(val.dptr, RSTRING_PTR(valstr), val.dsize) == 0)
 	    return rb_tainted_str_new(key.dptr, key.dsize);
     }
     return Qnil;
 }
 
 static VALUE
-fsdbm_indexes(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+fsdbm_select(VALUE obj)
 {
-    VALUE new;
-    int i;
+    VALUE new = rb_ary_new();
+    datum key, val;
+    DBM *dbm;
+    struct dbmdata *dbmp;
 
-    new = rb_ary_new2(argc);
-    for (i=0; i<argc; i++) {
-	rb_ary_push(new, fsdbm_fetch(obj, argv[i], Qnil));
-    }
-
-    return new;
-}
-
-static VALUE
-fsdbm_select(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
-{
-    VALUE new = rb_ary_new2(argc);
-    int i;
-
-    if (rb_block_given_p()) {
-        datum key, val;
-        DBM *dbm;
-        struct dbmdata *dbmp;
-
-	if (argc > 0) {
-	    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0)", argc);
+    GetDBM2(obj, dbmp, dbm);
+    for (key = sdbm_firstkey(dbm); key.dptr; key = sdbm_nextkey(dbm)) {
+	VALUE assoc, v;
+	val = sdbm_fetch(dbm, key);
+	assoc = rb_assoc_new(rb_tainted_str_new(key.dptr, key.dsize),
+			     rb_tainted_str_new(val.dptr, val.dsize));
+	v = rb_yield(assoc);
+	if (RTEST(v)) {
+	    rb_ary_push(new, assoc);
 	}
-        GetDBM2(obj, dbmp, dbm);
-        for (key = sdbm_firstkey(dbm); key.dptr; key = sdbm_nextkey(dbm)) {
-            VALUE assoc, v;
-            val = sdbm_fetch(dbm, key);
-            assoc = rb_assoc_new(rb_tainted_str_new(key.dptr, key.dsize),
-                                 rb_tainted_str_new(val.dptr, val.dsize));
-	    v = rb_yield(assoc);
-	    if (RTEST(v)) {
-		rb_ary_push(new, assoc);
-	    }
-	    GetDBM2(obj, dbmp, dbm);
-        }
-    }
-    else {
-	rb_warn("SDBM#select(index..) is deprecated; use SDBM#values_at");
-
-        for (i=0; i<argc; i++) {
-            rb_ary_push(new, fsdbm_fetch(obj, argv[i], Qnil));
-        }
+	GetDBM2(obj, dbmp, dbm);
     }
 
     return new;
 }
 
 static VALUE
-fsdbm_values_at(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+fsdbm_values_at(int argc, VALUE *argv, VALUE obj)
 {
     VALUE new = rb_ary_new2(argc);
     int i;
@@ -289,16 +234,14 @@ fsdbm_values_at(argc, argv, obj)
 }
 
 static void
-fdbm_modify(obj)
-    VALUE obj;
+fdbm_modify(VALUE obj)
 {
     rb_secure(4);
     if (OBJ_FROZEN(obj)) rb_error_frozen("SDBM");
 }
 
 static VALUE
-fsdbm_delete(obj, keystr)
-    VALUE obj, keystr;
+fsdbm_delete(VALUE obj, VALUE keystr)
 {
     datum key, value;
     struct dbmdata *dbmp;
@@ -307,8 +250,8 @@ fsdbm_delete(obj, keystr)
 
     fdbm_modify(obj);
     StringValue(keystr);
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
+    key.dptr = RSTRING_PTR(keystr);
+    key.dsize = RSTRING_LEN(keystr);
 
     GetDBM2(obj, dbmp, dbm);
     dbmp->di_size = -1;
@@ -333,8 +276,7 @@ fsdbm_delete(obj, keystr)
 }
 
 static VALUE
-fsdbm_shift(obj)
-    VALUE obj;
+fsdbm_shift(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -357,8 +299,7 @@ fsdbm_shift(obj)
 }
 
 static VALUE
-fsdbm_delete_if(obj)
-    VALUE obj;
+fsdbm_delete_if(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -381,24 +322,23 @@ fsdbm_delete_if(obj)
 	GetDBM2(obj, dbmp, dbm);
     }
 
-    for (i = 0; i < RARRAY(ary)->len; i++) {
-	keystr = RARRAY(ary)->ptr[i];
+    for (i = 0; i < RARRAY_LEN(ary); i++) {
+	keystr = RARRAY_PTR(ary)[i];
 	StringValue(keystr);
-	key.dptr = RSTRING(keystr)->ptr;
-	key.dsize = RSTRING(keystr)->len;
+	key.dptr = RSTRING_PTR(keystr);
+	key.dsize = RSTRING_LEN(keystr);
 	if (sdbm_delete(dbm, key)) {
 	    rb_raise(rb_eDBMError, "sdbm_delete failed");
 	}
     }
     if (status) rb_jump_tag(status);
-    if (n > 0) dbmp->di_size = n - RARRAY(ary)->len;
+    if (n > 0) dbmp->di_size = n - RARRAY_LEN(ary);
 
     return obj;
 }
 
 static VALUE
-fsdbm_clear(obj)
-    VALUE obj;
+fsdbm_clear(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -418,8 +358,7 @@ fsdbm_clear(obj)
 }
 
 static VALUE
-fsdbm_invert(obj)
-    VALUE obj;
+fsdbm_invert(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -437,49 +376,8 @@ fsdbm_invert(obj)
     return hash;
 }
 
-static VALUE each_pair _((VALUE));
-
 static VALUE
-each_pair(obj)
-    VALUE obj;
-{
-    return rb_funcall(obj, rb_intern("each_pair"), 0, 0);
-}
-
-static VALUE fsdbm_store _((VALUE,VALUE,VALUE));
-
-static VALUE
-update_i(pair, dbm)
-    VALUE pair, dbm;
-{
-    Check_Type(pair, T_ARRAY);
-    if (RARRAY(pair)->len < 2) {
-	rb_raise(rb_eArgError, "pair must be [key, value]");
-    }
-    fsdbm_store(dbm, RARRAY(pair)->ptr[0], RARRAY(pair)->ptr[1]);
-    return Qnil;
-}
-
-static VALUE
-fsdbm_update(obj, other)
-    VALUE obj, other;
-{
-    rb_iterate(each_pair, other, update_i, obj);
-    return obj;
-}
-
-static VALUE
-fsdbm_replace(obj, other)
-    VALUE obj, other;
-{
-    fsdbm_clear(obj);
-    rb_iterate(each_pair, other, update_i, obj);
-    return obj;
-}
-
-static VALUE
-fsdbm_store(obj, keystr, valstr)
-    VALUE obj, keystr, valstr;
+fsdbm_store(VALUE obj, VALUE keystr, VALUE valstr)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -494,11 +392,11 @@ fsdbm_store(obj, keystr, valstr)
     StringValue(keystr);
     StringValue(valstr);
 
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
+    key.dptr = RSTRING_PTR(keystr);
+    key.dsize = RSTRING_LEN(keystr);
 
-    val.dptr = RSTRING(valstr)->ptr;
-    val.dsize = RSTRING(valstr)->len;
+    val.dptr = RSTRING_PTR(valstr);
+    val.dsize = RSTRING_LEN(valstr);
 
     GetDBM2(obj, dbmp, dbm);
     dbmp->di_size = -1;
@@ -514,8 +412,33 @@ fsdbm_store(obj, keystr, valstr)
 }
 
 static VALUE
-fsdbm_length(obj)
-    VALUE obj;
+update_i(VALUE pair, VALUE dbm)
+{
+    Check_Type(pair, T_ARRAY);
+    if (RARRAY_LEN(pair) < 2) {
+	rb_raise(rb_eArgError, "pair must be [key, value]");
+    }
+    fsdbm_store(dbm, RARRAY_PTR(pair)[0], RARRAY_PTR(pair)[1]);
+    return Qnil;
+}
+
+static VALUE
+fsdbm_update(VALUE obj, VALUE other)
+{
+    rb_block_call(other, rb_intern("each_pair"), 0, 0, update_i, obj);
+    return obj;
+}
+
+static VALUE
+fsdbm_replace(VALUE obj, VALUE other)
+{
+    fsdbm_clear(obj);
+    rb_block_call(other, rb_intern("each_pair"), 0, 0, update_i, obj);
+    return obj;
+}
+
+static VALUE
+fsdbm_length(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -534,8 +457,7 @@ fsdbm_length(obj)
 }
 
 static VALUE
-fsdbm_empty_p(obj)
-    VALUE obj;
+fsdbm_empty_p(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -558,8 +480,7 @@ fsdbm_empty_p(obj)
 }
 
 static VALUE
-fsdbm_each_value(obj)
-    VALUE obj;
+fsdbm_each_value(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -575,8 +496,7 @@ fsdbm_each_value(obj)
 }
 
 static VALUE
-fsdbm_each_key(obj)
-    VALUE obj;
+fsdbm_each_key(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -591,8 +511,7 @@ fsdbm_each_key(obj)
 }
 
 static VALUE
-fsdbm_each_pair(obj)
-    VALUE obj;
+fsdbm_each_pair(VALUE obj)
 {
     datum key, val;
     DBM *dbm;
@@ -612,8 +531,7 @@ fsdbm_each_pair(obj)
 }
 
 static VALUE
-fsdbm_keys(obj)
-    VALUE obj;
+fsdbm_keys(VALUE obj)
 {
     datum key;
     struct dbmdata *dbmp;
@@ -630,8 +548,7 @@ fsdbm_keys(obj)
 }
 
 static VALUE
-fsdbm_values(obj)
-    VALUE obj;
+fsdbm_values(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -649,16 +566,15 @@ fsdbm_values(obj)
 }
 
 static VALUE
-fsdbm_has_key(obj, keystr)
-    VALUE obj, keystr;
+fsdbm_has_key(VALUE obj, VALUE keystr)
 {
     datum key, val;
     struct dbmdata *dbmp;
     DBM *dbm;
 
     StringValue(keystr);
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
+    key.dptr = RSTRING_PTR(keystr);
+    key.dsize = RSTRING_LEN(keystr);
 
     GetDBM2(obj, dbmp, dbm);
     val = sdbm_fetch(dbm, key);
@@ -667,30 +583,28 @@ fsdbm_has_key(obj, keystr)
 }
 
 static VALUE
-fsdbm_has_value(obj, valstr)
-    VALUE obj, valstr;
+fsdbm_has_value(VALUE obj, VALUE valstr)
 {
     datum key, val;
     struct dbmdata *dbmp;
     DBM *dbm;
 
     StringValue(valstr);
-    val.dptr = RSTRING(valstr)->ptr;
-    val.dsize = RSTRING(valstr)->len;
+    val.dptr = RSTRING_PTR(valstr);
+    val.dsize = RSTRING_LEN(valstr);
 
     GetDBM2(obj, dbmp, dbm);
     for (key = sdbm_firstkey(dbm); key.dptr; key = sdbm_nextkey(dbm)) {
 	val = sdbm_fetch(dbm, key);
-	if (val.dsize == RSTRING(valstr)->len &&
-	    memcmp(val.dptr, RSTRING(valstr)->ptr, val.dsize) == 0)
+	if (val.dsize == RSTRING_LEN(valstr) &&
+	    memcmp(val.dptr, RSTRING_PTR(valstr), val.dsize) == 0)
 	    return Qtrue;
     }
     return Qfalse;
 }
 
 static VALUE
-fsdbm_to_a(obj)
-    VALUE obj;
+fsdbm_to_a(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -709,8 +623,7 @@ fsdbm_to_a(obj)
 }
 
 static VALUE
-fsdbm_to_hash(obj)
-    VALUE obj;
+fsdbm_to_hash(VALUE obj)
 {
     datum key, val;
     struct dbmdata *dbmp;
@@ -729,8 +642,7 @@ fsdbm_to_hash(obj)
 }
 
 static VALUE
-fsdbm_reject(obj)
-    VALUE obj;
+fsdbm_reject(VALUE obj)
 {
     return rb_hash_delete_if(fsdbm_to_hash(obj));
 }
@@ -753,9 +665,7 @@ Init_sdbm()
     rb_define_method(rb_cDBM, "[]=", fsdbm_store, 2);
     rb_define_method(rb_cDBM, "store", fsdbm_store, 2);
     rb_define_method(rb_cDBM, "index",  fsdbm_index, 1);
-    rb_define_method(rb_cDBM, "indexes",  fsdbm_indexes, -1);
-    rb_define_method(rb_cDBM, "indices",  fsdbm_indexes, -1);
-    rb_define_method(rb_cDBM, "select",  fsdbm_select, -1);
+    rb_define_method(rb_cDBM, "select",  fsdbm_select, 0);
     rb_define_method(rb_cDBM, "values_at",  fsdbm_values_at, -1);
     rb_define_method(rb_cDBM, "length", fsdbm_length, 0);
     rb_define_method(rb_cDBM, "size", fsdbm_length, 0);
