@@ -2,8 +2,7 @@
 
   hash.c -
 
-  $Author: matz $
-  $Date: 2007-11-30 14:26:59 +0900 (Fri, 30 Nov 2007) $
+  $Author: naruse $
   created at: Mon Nov 22 18:51:18 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -54,7 +53,7 @@ rb_any_cmp(VALUE a, VALUE b)
     }
     if (TYPE(a) == T_STRING && RBASIC(a)->klass == rb_cString &&
 	TYPE(b) == T_STRING && RBASIC(b)->klass == rb_cString) {
-	return rb_str_cmp(a, b);
+	return rb_str_hash_cmp(a, b);
     }
     if (a == Qundef || b == Qundef) return -1;
     if (SYMBOL_P(a) && SYMBOL_P(b)) {
@@ -1170,7 +1169,7 @@ inspect_hash(VALUE hash, VALUE dummy, int recur)
 {
     VALUE str;
 
-    if (recur) return rb_str_new2("{...}");
+    if (recur) return rb_usascii_str_new2("{...}");
     str = rb_str_buf_new2("{");
     rb_hash_foreach(hash, inspect_i, str);
     rb_str_buf_cat2(str, "}");
@@ -1194,7 +1193,7 @@ static VALUE
 rb_hash_inspect(VALUE hash)
 {
     if (RHASH_EMPTY_P(hash))
-	return rb_str_new2("{}");
+	return rb_usascii_str_new2("{}");
     return rb_exec_recursive(inspect_hash, hash, 0);
 }
 
@@ -1536,7 +1535,7 @@ rb_hash_update_block_i(VALUE key, VALUE value, VALUE hash)
  *  Adds the contents of <i>other_hash</i> to <i>hsh</i>.  If no
  *  block is specified entries with duplicate keys are overwritten
  *  with the values from <i>other_hash</i>, otherwise the value
- *  of each duplicate key is detemined by calling the block with
+ *  of each duplicate key is determined by calling the block with
  *  the key, its value in <i>hsh</i> and its value in <i>other_hash</i>.
  *
  *     h1 = { "a" => 100, "b" => 200 }
@@ -1661,7 +1660,7 @@ rb_hash_rassoc(VALUE hash, VALUE obj)
  *  hash. That is, for every key or value that is an array, extract
  *  its elements into the new array.  Unlike Array#flatten, this
  *  method does not flatten recursively by default.  If the optional
- *  <i>level</i> argument determins the level of recursion to flatten.
+ *  <i>level</i> argument determines the level of recursion to flatten.
  *
  *     a =  {1=> "one", 2 => [2,"two"], 3 => "three"}
  *     a.flatten    # => [1, "one", 2, [2, "two"], 3, "three"]
@@ -1671,9 +1670,14 @@ rb_hash_rassoc(VALUE hash, VALUE obj)
 static VALUE
 rb_hash_flatten(int argc, VALUE *argv, VALUE hash)
 {
-    VALUE ary;
+    VALUE ary, tmp;
 
     ary = rb_hash_to_a(hash);
+    if (argc == 0) {
+	argc = 1;
+	tmp = INT2FIX(1);
+	argv = &tmp;
+    }
     rb_funcall2(ary, rb_intern("flatten!"), argc, argv);
     return ary;
 }
@@ -1781,7 +1785,7 @@ env_delete(VALUE obj, VALUE name)
 
 	ruby_setenv(nam, 0);
 #ifdef ENV_IGNORECASE
-	if (strcasecmp(nam, PATH_ENV) == 0)
+	if (STRCASECMP(nam, PATH_ENV) == 0)
 #else
 	if (strcmp(nam, PATH_ENV) == 0)
 #endif
@@ -1817,7 +1821,7 @@ rb_f_getenv(VALUE obj, VALUE name)
     env = getenv(nam);
     if (env) {
 #ifdef ENV_IGNORECASE
-	if (strcasecmp(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
+	if (STRCASECMP(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
 #else
 	if (strcmp(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
 #endif
@@ -1859,7 +1863,7 @@ env_fetch(int argc, VALUE *argv)
 	return if_none;
     }
 #ifdef ENV_IGNORECASE
-    if (strcasecmp(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
+    if (STRCASECMP(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
 #else
     if (strcmp(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
 #endif
@@ -1893,7 +1897,7 @@ envix(const char *nam)
     for (i = 0; env[i]; i++) {
 	if (
 #ifdef ENV_IGNORECASE
-	    strncasecmp(env[i],nam,len) == 0
+	    STRNCASECMP(env[i],nam,len) == 0
 #else
 	    memcmp(env[i],nam,len) == 0
 #endif
@@ -2014,7 +2018,7 @@ env_aset(VALUE obj, VALUE nm, VALUE val)
 
     ruby_setenv(name, value);
 #ifdef ENV_IGNORECASE
-    if (strcasecmp(name, PATH_ENV) == 0) {
+    if (STRCASECMP(name, PATH_ENV) == 0) {
 #else
     if (strcmp(name, PATH_ENV) == 0) {
 #endif
@@ -2218,7 +2222,7 @@ env_clear(void)
 static VALUE
 env_to_s(void)
 {
-    return rb_str_new2("ENV");
+    return rb_usascii_str_new2("ENV");
 }
 
 static VALUE
@@ -2373,8 +2377,9 @@ env_rassoc(VALUE dmy, VALUE obj)
 	if (s++) {
 	    long len = strlen(s);
 	    if (RSTRING_LEN(obj) == len && strncmp(s, RSTRING_PTR(obj), len) == 0) {
+		VALUE result = rb_assoc_new(rb_tainted_str_new(*env, s-*env-1), obj);
 		FREE_ENVIRON(environ);
-		return rb_assoc_new(rb_tainted_str_new(*env, s-*env-1), obj);
+		return result;
 	    }
 	}
 	env++;
