@@ -1,9 +1,8 @@
 #
 #   irb/workspace-binding.rb - 
-#   	$Release Version: 0.9$
-#   	$Revision: 1.5.2.1 $
-#   	$Date: 2004/04/18 23:20:32 $
-#   	by Keiju ISHITSUKA(keiju@ishitsuka.com)
+#   	$Release Version: 0.9.5$
+#   	$Revision: 15442 $
+#   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
 #
@@ -96,7 +95,7 @@ EOF
 	return nil if bt =~ /irb\/.*\.rb/
       when 3
 	return nil if bt =~ /irb\/.*\.rb/
-	bt.sub!(/:\s*in `irb_binding'/){""} 
+	bt.sub!(/:\s*in `irb_binding'/, '')
       end
       bt
     end
