@@ -3,7 +3,6 @@
 
 require "rdoc/code_objects"
 require "rdoc/parsers/parserfactory"
-require "rdoc/options"
 require "rdoc/rdoc"
 
 module RDoc
@@ -37,6 +36,7 @@ module RDoc
     "rb_cTime"             => "Time",
     "rb_cTrueClass"        => "TrueClass",
     "rb_cStruct"           => "Struct",
+    "rb_cVM"               => "VM",
     "rb_eException"        => "Exception",
     "rb_eStandardError"    => "StandardError",
     "rb_eSystemExit"       => "SystemExit",
@@ -138,7 +138,7 @@ module RDoc
   # Ruby function is in the same source file as the rb_define_method call.
   # If this isn't the case, add the comment 
   #
-  #    rb_define_method(....);  // in filename
+  #    rb_define_method(....);  // in: filename
   #
   # As an example, we might have an extension that defines multiple classes
   # in its Init_xxx method. We could document them using
@@ -166,23 +166,24 @@ module RDoc
 
   class C_Parser
 
-    attr_accessor :progress
+    attr_writer :progress
 
     extend ParserFactory
     parse_files_matching(/\.(?:([CcHh])\1?|c([+xp])\2|y)\z/)
 
+    @@enclosure_classes = {}
     @@known_bodies = {}
 
     # prepare to parse a C file
     def initialize(top_level, file_name, body, options, stats)
       @known_classes = KNOWN_CLASSES.dup
-      @body = handle_tab_width(handle_ifdefs_in(body))
       @options = options
+      @body = handle_tab_width(handle_ifdefs_in(body))
       @stats   = stats
       @top_level = top_level
       @classes = Hash.new
       @file_dir = File.dirname(file_name)
-      @progress = $stderr unless options.quiet
+      @progress = $stderr unless @options.quiet
     end
 
     # Extract the classes/modules and methods from a C file
@@ -233,7 +234,7 @@ module RDoc
       parent_name = @known_classes[parent] || parent
 
       if in_module
-        enclosure = @classes[in_module]
+        enclosure = @classes[in_module] || @@enclosure_classes[in_module]
         unless enclosure
           if enclosure = @known_classes[in_module]
             handle_class_module(in_module, (/^rb_m/ =~ in_module ? "module" : "class"),
@@ -261,6 +262,7 @@ module RDoc
 
       find_class_comment(cm.full_name, cm)
       @classes[var_name] = cm
+      @@enclosure_classes[var_name] = cm
       @known_classes[var_name] = cm.full_name
     end
 
@@ -293,7 +295,7 @@ module RDoc
     def find_class_comment(class_name, class_meth)
       comment = nil
       if @body =~ %r{((?>/\*.*?\*/\s+))
-                     (static\s+)?void\s+Init_#{class_name}\s*(?:_\(\s*)?\(\s*(?:void\s*)?\)}xmi
+                     (static\s+)?void\s+Init_#{class_name}\s*(?:_\(\s*)?\(\s*(?:void\s*)\)}xmi
         comment = $1
       elsif @body =~ %r{Document-(class|module):\s#{class_name}\s*?\n((?>.*?\*/))}m
         comment = $2
@@ -625,8 +627,8 @@ module RDoc
     # Find the C code corresponding to a Ruby method
     def find_body(meth_name, meth_obj, body, quiet = false)
       case body
-      when %r{((?>/\*.*?\*/\s*))(?:static\s+)?VALUE\s+#{meth_name}
-              \s*(\(.*?\)).*?^}xm
+      when %r"((?>/\*.*?\*/\s*))(?:static\s+)?VALUE\s+#{meth_name}
+              \s*(\([^)]*\))\s*\{.*?^\}"xm
         comment, params = $1, $2
         body_text = $&
 
@@ -750,7 +752,7 @@ module RDoc
 
     def handle_tab_width(body)
       if /\t/ =~ body
-        tab_width = Options.instance.tab_width
+        tab_width = @options.tab_width
         body.split(/\n/).map do |line|
           1 while line.gsub!(/\t+/) { ' ' * (tab_width*$&.length - $`.length % tab_width)}  && $~ #`
           line
@@ -764,7 +766,7 @@ module RDoc
     # Removes #ifdefs that would otherwise confuse us
     
     def handle_ifdefs_in(body)
-      body.gsub(/^#ifdef HAVE_PROTOTYPES.*?#else.*?\n(.*?)#endif.*?\n/m) { $1 }
+      body.gsub(/^#ifdef HAVE_PROTOTYPES.*?#else.*?\n(.*?)#endif.*?\n/m, '\1')
     end
     
   end
