@@ -3,11 +3,6 @@ require 'fcntl'
 require 'timeout'
 require 'thread'
 
-begin
-  require 'securerandom'
-rescue LoadError
-end
-
 # Resolv is a thread-aware DNS resolver library written in Ruby.  Resolv can
 # handle multiple DNS requests concurrently without blocking.  The ruby
 # interpreter.
@@ -165,7 +160,7 @@ class Resolv
   # DNS::Hosts is a hostname resolver that uses the system hosts file.
 
   class Hosts
-    if /mswin32|mingw|bccwin/ =~ RUBY_PLATFORM
+    if /mswin32|cygwin|mingw|bccwin/ =~ RUBY_PLATFORM
       require 'win32/resolv'
       DefaultFileName = Win32::Resolv.get_hosts_path
     else
@@ -257,7 +252,7 @@ class Resolv
     end
 
     ##
-    # Iterates over all hostnames for +address+ retrieved from the hosts file.
+    # Iterates over all hostnames for +address+ retrived from the hosts file.
 
     def each_name(address, &proc)
       lazy_initialize
@@ -290,7 +285,7 @@ class Resolv
     UDPSize = 512
 
     ##
-    # Group of DNS resolver threads (obsolete)
+    # Group of DNS resolver threads
 
     DNSThreadGroup = ThreadGroup.new
 
@@ -335,6 +330,13 @@ class Resolv
       @mutex.synchronize {
         unless @initialized
           @config.lazy_initialize
+
+          if nameserver = @config.single?
+            @requester = Requester::ConnectedUDP.new(nameserver)
+          else
+            @requester = Requester::UnconnectedUDP.new
+          end
+
           @initialized = true
         end
       }
@@ -347,6 +349,8 @@ class Resolv
     def close
       @mutex.synchronize {
         if @initialized
+          @requester.close if @requester
+          @requester = nil
           @initialized = false
         end
       }
@@ -475,7 +479,7 @@ class Resolv
   
     def each_resource(name, typeclass, &proc)
       lazy_initialize
-      requester = make_requester
+      q = Queue.new
       senders = {}
       begin
         @config.resolv(name) {|candidate, tout, nameserver|
@@ -484,9 +488,11 @@ class Resolv
           msg.add_question(candidate, typeclass)
           unless sender = senders[[candidate, nameserver]]
             sender = senders[[candidate, nameserver]] =
-              requester.sender(msg, candidate, nameserver)
+              @requester.sender(msg, candidate, q, nameserver)
           end
-          reply, reply_name = requester.request(sender, tout)
+          sender.send
+          reply = reply_name = nil
+          timeout(tout, ResolvTimeout) { reply, reply_name = q.pop }
           case reply.rcode
           when RCode::NoError
             extract_resources(reply, reply_name, typeclass, &proc)
@@ -498,15 +504,7 @@ class Resolv
           end
         }
       ensure
-        requester.close
-      end
-    end
-
-    def make_requester # :nodoc:
-      if nameserver = @config.single?
-        Requester::ConnectedUDP.new(nameserver)
-      else
-        Requester::UnconnectedUDP.new
+        @requester.delete(q)
       end
     end
 
@@ -541,106 +539,45 @@ class Resolv
       }
     end
 
-    if defined? SecureRandom
-      def self.random(arg) # :nodoc:
-        begin
-          SecureRandom.random_number(arg)
-        rescue NotImplementedError
-          rand(arg)
-        end
-      end
-    else
-      def self.random(arg) # :nodoc:
-        rand(arg)
-      end
-    end
-
-
-    def self.rangerand(range) # :nodoc:
-      base = range.begin
-      len = range.end - range.begin
-      if !range.exclude_end?
-        len += 1
-      end
-      base + random(len)
-    end
-
-    RequestID = {}
-    RequestIDMutex = Mutex.new
-
-    def self.allocate_request_id(host, port) # :nodoc:
-      id = nil
-      RequestIDMutex.synchronize {
-        h = (RequestID[[host, port]] ||= {})
-        begin
-          id = rangerand(0x0000..0xffff)
-        end while h[id] 
-        h[id] = true
-      }
-      id
-    end
-
-    def self.free_request_id(host, port, id) # :nodoc:
-      RequestIDMutex.synchronize {
-        key = [host, port]
-        if h = RequestID[key]
-          h.delete id
-          if h.empty?
-            RequestID.delete key
-          end
-        end
-      }
-    end
-
-    def self.bind_random_port(udpsock, is_ipv6=false) # :nodoc:
-      begin
-        port = rangerand(1024..65535)
-        udpsock.bind(is_ipv6 ? "::" : "", port)
-      rescue Errno::EADDRINUSE
-        retry
-      end
-    end
-
     class Requester # :nodoc:
       def initialize
         @senders = {}
-        @sock = nil
-      end
-
-      def request(sender, tout)
-        timelimit = Time.now + tout
-        sender.send
-        while (now = Time.now) < timelimit
-          timeout = timelimit - now
-          if !IO.select([@sock], nil, nil, timeout)
-            raise ResolvTimeout
-          end
-          reply, from = recv_reply
-          begin
-            msg = Message.decode(reply)
-          rescue DecodeError
-            next # broken DNS message ignored
-          end
-          if s = @senders[[from,msg.id]]
-            break
-          else
-            # unexpected DNS message ignored
-          end
-        end
-        return msg, s.data
       end
 
       def close
-        sock = @sock
-        @sock = nil
-        sock.close if sock
+        thread, sock, @thread, @sock = @thread, @sock
+        begin
+          if thread
+            thread.kill
+            thread.join
+          end
+        ensure
+          sock.close if sock
+        end
+      end
+
+      def delete(arg)
+        case arg
+        when Sender
+          @senders.delete_if {|k, s| s == arg }
+        when Queue
+          @senders.delete_if {|k, s| s.queue == arg }
+        else
+          raise ArgumentError.new("neither Sender or Queue: #{arg}")
+        end
       end
 
       class Sender # :nodoc:
-        def initialize(msg, data, sock)
+        def initialize(msg, data, sock, queue)
           @msg = msg
           @data = data
           @sock = sock
+          @queue = queue
+        end
+        attr_reader :queue
+
+        def recv(msg)
+          @queue.push([msg, @data])
         end
       end
 
@@ -648,38 +585,45 @@ class Resolv
         def initialize
           super()
           @sock = UDPSocket.new
-          @sock.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC) if defined? Fcntl::F_SETFD
-          DNS.bind_random_port(@sock)
-        end
-
-        def recv_reply
-          reply, from = @sock.recvfrom(UDPSize)
-          return reply, [from[3],from[1]]
-        end
-
-        def sender(msg, data, host, port=Port)
-          service = [host, port]
-          id = DNS.allocate_request_id(host, port)
-          request = msg.encode
-          request[0,2] = [id].pack('n')
-          return @senders[[service, id]] =
-            Sender.new(request, data, @sock, host, port)
-        end
-
-        def close
-          super
-          @senders.each_key {|service, id|
-            DNS.free_request_id(service[0], service[1], id)
+          @sock.fcntl(Fcntl::F_SETFD, 1) if defined? Fcntl::F_SETFD
+          @id = {}
+          @id.default = -1
+          @thread = Thread.new {
+            DNSThreadGroup.add Thread.current
+            loop {
+              reply, from = @sock.recvfrom(UDPSize)
+              msg = begin
+                Message.decode(reply)
+              rescue DecodeError
+                STDERR.print("DNS message decoding error: #{reply.inspect}\n")
+                next
+              end
+              if s = @senders[[[from[3],from[1]],msg.id]]
+                s.recv msg
+              else
+                #STDERR.print("non-handled DNS message: #{msg.inspect} from #{from.inspect}\n")
+              end
+            }
           }
         end
 
+        def sender(msg, data, queue, host, port=Port)
+          service = [host, port]
+          id = Thread.exclusive {
+            @id[service] = (@id[service] + 1) & 0xffff
+          }
+          request = msg.encode
+          request[0,2] = [id].pack('n')
+          return @senders[[service, id]] =
+            Sender.new(request, data, @sock, host, port, queue)
+        end
+
         class Sender < Requester::Sender # :nodoc:
-          def initialize(msg, data, sock, host, port)
-            super(msg, data, sock)
+          def initialize(msg, data, sock, host, port, queue)
+            super(msg, data, sock, queue)
             @host = host
             @port = port
           end
-          attr_reader :data
 
           def send
             @sock.send(@msg, 0, @host, @port)
@@ -692,40 +636,43 @@ class Resolv
           super()
           @host = host
           @port = port
-          is_ipv6 = host.index(':')
-          @sock = UDPSocket.new(is_ipv6 ? Socket::AF_INET6 : Socket::AF_INET)
-          DNS.bind_random_port(@sock, is_ipv6)
+          @sock = UDPSocket.new(host.index(':') ? Socket::AF_INET6 : Socket::AF_INET)
           @sock.connect(host, port)
-          @sock.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC) if defined? Fcntl::F_SETFD
+          @sock.fcntl(Fcntl::F_SETFD, 1) if defined? Fcntl::F_SETFD
+          @id = -1
+          @thread = Thread.new {
+            DNSThreadGroup.add Thread.current
+            loop {
+              reply = @sock.recv(UDPSize)
+              msg = begin
+                Message.decode(reply)
+              rescue DecodeError
+                STDERR.print("DNS message decoding error: #{reply.inspect}")
+                next
+              end
+              if s = @senders[msg.id]
+                s.recv msg
+              else
+                #STDERR.print("non-handled DNS message: #{msg.inspect}")
+              end
+            }
+          }
         end
 
-        def recv_reply
-          reply = @sock.recv(UDPSize)
-          return reply, nil
-        end
-
-        def sender(msg, data, host=@host, port=@port)
+        def sender(msg, data, queue, host=@host, port=@port)
           unless host == @host && port == @port
             raise RequestError.new("host/port don't match: #{host}:#{port}")
           end
-          id = DNS.allocate_request_id(@host, @port)
+          id = Thread.exclusive { @id = (@id + 1) & 0xffff }
           request = msg.encode
           request[0,2] = [id].pack('n')
-          return @senders[[nil,id]] = Sender.new(request, data, @sock)
-        end
-
-        def close
-          super
-          @senders.each_key {|from, id|
-            DNS.free_request_id(@host, @port, id)
-          }
+          return @senders[id] = Sender.new(request, data, @sock, queue)
         end
 
         class Sender < Requester::Sender # :nodoc:
           def send
             @sock.send(@msg, 0)
           end
-          attr_reader :data
         end
       end
 
@@ -734,25 +681,39 @@ class Resolv
           super()
           @host = host
           @port = port
-          @sock = TCPSocket.new(@host, @port)
-          @sock.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC) if defined? Fcntl::F_SETFD
+          @sock = TCPSocket.new
+          @sock.connect(host, port)
+          @sock.fcntl(Fcntl::F_SETFD, 1) if defined? Fcntl::F_SETFD
+          @id = -1
           @senders = {}
+          @thread = Thread.new {
+            DNSThreadGroup.add Thread.current
+            loop {
+              len = @sock.read(2).unpack('n')
+              reply = @sock.read(len)
+              msg = begin
+                Message.decode(reply)
+              rescue DecodeError
+                STDERR.print("DNS message decoding error: #{reply.inspect}")
+                next
+              end
+              if s = @senders[msg.id]
+                s.push msg
+              else
+                #STDERR.print("non-handled DNS message: #{msg.inspect}")
+              end
+            }
+          }
         end
 
-        def recv_reply
-          len = @sock.read(2).unpack('n')[0]
-          reply = @sock.read(len)
-          return reply, nil
-        end
-
-        def sender(msg, data, host=@host, port=@port)
+        def sender(msg, data, queue, host=@host, port=@port)
           unless host == @host && port == @port
             raise RequestError.new("host/port don't match: #{host}:#{port}")
           end
-          id = DNS.allocate_request_id(@host, @port)
+          id = Thread.exclusive { @id = (@id + 1) & 0xffff }
           request = msg.encode
           request[0,2] = [request.length, id].pack('nn')
-          return @senders[[nil,id]] = Sender.new(request, data, @sock)
+          return @senders[id] = Sender.new(request, data, @sock, queue)
         end
 
         class Sender < Requester::Sender # :nodoc:
@@ -760,14 +721,6 @@ class Resolv
             @sock.print(@msg)
             @sock.flush
           end
-          attr_reader :data
-        end
-
-        def close
-          super
-          @senders.each_key {|from,id|
-            DNS.free_request_id(@host, @port, id)
-          }
         end
       end
 
@@ -824,7 +777,6 @@ class Resolv
           config_hash = Config.parse_resolv_conf(filename)
         else
           if /mswin32|cygwin|mingw|bccwin/ =~ RUBY_PLATFORM
-            require 'win32/resolv'
             search, nameserver = Win32::Resolv.get_resolv_info
             config_hash = {}
             config_hash[:nameserver] = nameserver if nameserver
@@ -1377,6 +1329,7 @@ class Resolv
         def get_unpack(template)
           len = 0
           template.each_byte {|byte|
+            byte = "%c" % byte
             case byte
             when ?c, ?C
               len += 1
@@ -1395,7 +1348,7 @@ class Resolv
         end
 
         def get_string
-          len = @data[@index]
+          len = @data[@index].ord
           raise DecodeError.new("limit exceeded") if @limit < @index + 1 + len
           d = @data[@index + 1, len]
           @index += 1 + len
@@ -1418,7 +1371,7 @@ class Resolv
           limit = @index if !limit || @index < limit
           d = []
           while true
-            case @data[@index]
+            case @data[@index].ord
             when 0
               @index += 1
               return d
@@ -1453,7 +1406,9 @@ class Resolv
           name = self.get_name
           type, klass, ttl = self.get_unpack('nnN')
           typeclass = Resource.get_class(type, klass)
-          return name, ttl, self.get_length16 {typeclass.decode_rdata(self)}
+          res = self.get_length16 { typeclass.decode_rdata self }
+          res.instance_variable_set :@ttl, ttl
+          return name, ttl, res
         end
       end
     end
@@ -1492,10 +1447,16 @@ class Resolv
       end
 
       def ==(other) # :nodoc:
-        return self.class == other.class &&
-          self.instance_variables == other.instance_variables &&
-          self.instance_variables.collect {|name| self.instance_eval name} ==
-            other.instance_variables.collect {|name| other.instance_eval name}
+        return false unless self.class == other.class
+        s_ivars = self.instance_variables
+        s_ivars.sort!
+        s_ivars.delete "@ttl"
+        o_ivars = other.instance_variables
+        o_ivars.sort!
+        o_ivars.delete "@ttl"
+        return s_ivars == o_ivars &&
+          s_ivars.collect {|name| self.instance_variable_get name} ==
+            o_ivars.collect {|name| other.instance_variable_get name}
       end
 
       def eql?(other) # :nodoc:
@@ -1504,8 +1465,10 @@ class Resolv
 
       def hash # :nodoc:
         h = 0
-        self.instance_variables.each {|name|
-          h ^= self.instance_eval("#{name}.hash")
+        vars = self.instance_variables
+        vars.delete "@ttl"
+        vars.each {|name|
+          h ^= self.instance_variable_get(name).hash
         }
         return h
       end
@@ -2062,7 +2025,7 @@ class Resolv
     end
 
     ##
-    # A String representation of this IPv4 address.
+    # A String reperesentation of this IPv4 address.
 
     ##
     # The raw IPv4 address as a String.
