@@ -2,8 +2,7 @@
 
   object.c -
 
-  $Author: akr $
-  $Date: 2007-12-24 16:52:39 +0900 (Mon, 24 Dec 2007) $
+  $Author: naruse $
   created at: Thu Jul 15 12:01:24 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -20,6 +19,7 @@
 #include <errno.h>
 #include <ctype.h>
 #include <math.h>
+#include <float.h>
 
 VALUE rb_cBasicObject;
 VALUE rb_mKernel;
@@ -166,21 +166,23 @@ init_copy(VALUE dest, VALUE obj)
     rb_gc_copy_finalizer(dest, obj);
     switch (TYPE(obj)) {
       case T_OBJECT:
-        if (!(RBASIC(dest)->flags & ROBJECT_EMBED) && ROBJECT_PTR(dest)) {
-            xfree(ROBJECT_PTR(dest));
-            ROBJECT(dest)->as.heap.ptr = 0;
-            ROBJECT(dest)->as.heap.len = 0;
+        if (!(RBASIC(dest)->flags & ROBJECT_EMBED) && ROBJECT_IVPTR(dest)) {
+            xfree(ROBJECT_IVPTR(dest));
+            ROBJECT(dest)->as.heap.ivptr = 0;
+            ROBJECT(dest)->as.heap.numiv = 0;
+            ROBJECT(dest)->as.heap.iv_index_tbl = 0;
         }
         if (RBASIC(obj)->flags & ROBJECT_EMBED) {
             MEMCPY(ROBJECT(dest)->as.ary, ROBJECT(obj)->as.ary, VALUE, ROBJECT_EMBED_LEN_MAX);
             RBASIC(dest)->flags |= ROBJECT_EMBED;
         }
         else {
-            long len = ROBJECT(obj)->as.heap.len;
+            long len = ROBJECT(obj)->as.heap.numiv;
             VALUE *ptr = ALLOC_N(VALUE, len);
-            MEMCPY(ptr, ROBJECT(obj)->as.heap.ptr, VALUE, len);
-            ROBJECT(dest)->as.heap.ptr = ptr;
-            ROBJECT(dest)->as.heap.len = len;
+            MEMCPY(ptr, ROBJECT(obj)->as.heap.ivptr, VALUE, len);
+            ROBJECT(dest)->as.heap.ivptr = ptr;
+            ROBJECT(dest)->as.heap.numiv = len;
+            ROBJECT(dest)->as.heap.iv_index_tbl = ROBJECT(obj)->as.heap.iv_index_tbl;
             RBASIC(dest)->flags &= ~ROBJECT_EMBED;
         }
         break;
@@ -372,8 +374,8 @@ rb_obj_inspect(VALUE obj)
 
     if (TYPE(obj) == T_OBJECT) {
         int has_ivar = 0;
-        VALUE *ptr = ROBJECT_PTR(obj);
-        long len = ROBJECT_LEN(obj);
+        VALUE *ptr = ROBJECT_IVPTR(obj);
+        long len = ROBJECT_NUMIV(obj);
         long i;
 
         for (i = 0; i < len; i++) {
@@ -806,7 +808,7 @@ nil_to_f(VALUE obj)
 static VALUE
 nil_to_s(VALUE obj)
 {
-    return rb_str_new2("");
+    return rb_usascii_str_new(0, 0);
 }
 
 /*
@@ -836,7 +838,7 @@ nil_to_a(VALUE obj)
 static VALUE
 nil_inspect(VALUE obj)
 {
-    return rb_str_new2("nil");
+    return rb_usascii_str_new2("nil");
 }
 
 /***********************************************************************
@@ -859,7 +861,7 @@ nil_inspect(VALUE obj)
 static VALUE
 true_to_s(VALUE obj)
 {
-    return rb_str_new2("true");
+    return rb_usascii_str_new2("true");
 }
 
 
@@ -936,7 +938,7 @@ true_xor(VALUE obj, VALUE obj2)
 static VALUE
 false_to_s(VALUE obj)
 {
-    return rb_str_new2("false");
+    return rb_usascii_str_new2("false");
 }
 
 /*
@@ -1090,7 +1092,7 @@ static VALUE
 rb_mod_to_s(VALUE klass)
 {
     if (FL_TEST(klass, FL_SINGLETON)) {
-	VALUE s = rb_str_new2("#<");
+	VALUE s = rb_usascii_str_new2("#<");
 	VALUE v = rb_iv_get(klass, "__attached__");
 
 	rb_str_cat2(s, "Class:");
@@ -1940,7 +1942,10 @@ rb_check_convert_type(VALUE val, int type, const char *tname, const char *method
 static VALUE
 rb_to_integer(VALUE val, const char *method)
 {
-    VALUE v = convert_type(val, "Integer", method, Qtrue);
+    VALUE v;
+
+    if (FIXNUM_P(val)) return val;
+    v = convert_type(val, "Integer", method, Qtrue);
     if (!rb_obj_is_kind_of(v, rb_cInteger)) {
 	char *cname = rb_obj_classname(val);
 	rb_raise(rb_eTypeError, "can't convert %s to Integer (%s#%s gives %s)",
@@ -1952,7 +1957,10 @@ rb_to_integer(VALUE val, const char *method)
 VALUE
 rb_check_to_integer(VALUE val, const char *method)
 {
-    VALUE v = convert_type(val, "Integer", method, Qfalse);
+    VALUE v;
+
+    if (FIXNUM_P(val)) return val;
+    v = convert_type(val, "Integer", method, Qfalse);
     if (!rb_obj_is_kind_of(v, rb_cInteger)) {
 	return Qnil;
     }
@@ -2049,15 +2057,16 @@ rb_cstr_to_dbl(const char *p, int badcheck)
 	return d;
     }
     if (*end) {
-	char *buf = ALLOCA_N(char, strlen(p)+1);
+	char buf[DBL_DIG * 4 + 10];
 	char *n = buf;
+	char *e = buf + sizeof(buf) - 1;
 
-	while (p < end) *n++ = *p++;
-	while (*p) {
+	while (p < end && n < e) *n++ = *p++;
+	while (n < e && *p) {
 	    if (*p == '_') {
 		/* remove underscores between digits */
-		    if (n == buf || !ISDIGIT(n[-1])) goto bad;
-		    while (*++p == '_');
+		if (n == buf || !ISDIGIT(n[-1])) goto bad;
+		while (*++p == '_');
 		if (!ISDIGIT(*p)) {
 		    if (badcheck) goto bad;
 		    break;
