@@ -2,25 +2,21 @@
 
   struct.c -
 
-  $Author: shyouhei $
-  $Date: 2007-08-22 09:56:18 +0900 (Wed, 22 Aug 2007) $
+  $Author: akr $
   created at: Tue Mar 22 18:44:30 JST 1995
 
-  Copyright (C) 1993-2003 Yukihiro Matsumoto
+  Copyright (C) 1993-2007 Yukihiro Matsumoto
 
 **********************************************************************/
 
-#include "ruby.h"
-#include "env.h"
+#include "ruby/ruby.h"
 
 VALUE rb_cStruct;
 
-static VALUE struct_alloc _((VALUE));
+static VALUE struct_alloc(VALUE);
 
 VALUE
-rb_struct_iv_get(c, name)
-    VALUE c;
-    char *name;
+rb_struct_iv_get(VALUE c, const char *name)
 {
     ID id;
 
@@ -28,49 +24,49 @@ rb_struct_iv_get(c, name)
     for (;;) {
 	if (rb_ivar_defined(c, id))
 	    return rb_ivar_get(c, id);
-	c = RCLASS(c)->super;
+	c = RCLASS_SUPER(c);
 	if (c == 0 || c == rb_cStruct)
 	    return Qnil;
     }
 }
 
 VALUE
-rb_struct_s_members(klass)
-    VALUE klass;
+rb_struct_s_members(VALUE klass)
 {
     VALUE members = rb_struct_iv_get(klass, "__members__");
 
     if (NIL_P(members)) {
-	rb_bug("non-initialized struct");
+	rb_raise(rb_eTypeError, "uninitialized struct");
+    }
+    if (TYPE(members) != T_ARRAY) {
+	rb_raise(rb_eTypeError, "corrupted struct");
     }
     return members;
 }
 
 VALUE
-rb_struct_members(s)
-    VALUE s;
+rb_struct_members(VALUE s)
 {
     VALUE members = rb_struct_s_members(rb_obj_class(s));
 
-    if (RSTRUCT(s)->len != RARRAY(members)->len) {
-	rb_raise(rb_eTypeError, "struct size differs (%d required %d given)",
-		 RARRAY(members)->len, RSTRUCT(s)->len);
+    if (RSTRUCT_LEN(s) != RARRAY_LEN(members)) {
+	rb_raise(rb_eTypeError, "struct size differs (%ld required %ld given)",
+		 RARRAY_LEN(members), RSTRUCT_LEN(s));
     }
     return members;
 }
 
 static VALUE
-rb_struct_s_members_m(klass)
-    VALUE klass;
+rb_struct_s_members_m(VALUE klass)
 {
     VALUE members, ary;
     VALUE *p, *pend;
 
     members = rb_struct_s_members(klass);
-    ary = rb_ary_new2(RARRAY(members)->len);
-    p = RARRAY(members)->ptr; pend = p + RARRAY(members)->len;
+    ary = rb_ary_new2(RARRAY_LEN(members));
+    p = RARRAY_PTR(members); pend = p + RARRAY_LEN(members);
     while (p < pend) {
-	rb_ary_push(ary, rb_str_new2(rb_id2name(SYM2ID(*p))));
+	rb_ary_push(ary, *p);
 	p++;
     }
 
@@ -90,25 +86,22 @@ rb_struct_s_members_m(klass)
  */
 
 static VALUE
-rb_struct_members_m(obj)
-    VALUE obj;
+rb_struct_members_m(VALUE obj)
 {
     return rb_struct_s_members_m(rb_obj_class(obj));
 }
 
 VALUE
-rb_struct_getmember(obj, id)
-    VALUE obj;
-    ID id;
+rb_struct_getmember(VALUE obj, ID id)
 {
     VALUE members, slot;
     long i;
 
     members = rb_struct_members(obj);
     slot = ID2SYM(id);
-    for (i=0; i<RARRAY(members)->len; i++) {
-	if (RARRAY(members)->ptr[i] == slot) {
-	    return RSTRUCT(obj)->ptr[i];
+    for (i=0; i<RARRAY_LEN(members); i++) {
+	if (RARRAY_PTR(members)[i] == slot) {
+	    return RSTRUCT_PTR(obj)[i];
 	}
     }
     rb_name_error(id, "%s is not struct member", rb_id2name(id));
@@ -116,24 +109,25 @@ rb_struct_getmember(obj, id)
 }
 
 static VALUE
-rb_struct_ref(obj)
-    VALUE obj;
+rb_struct_ref(VALUE obj)
 {
-    return rb_struct_getmember(obj, ruby_frame->orig_func);
+    return rb_struct_getmember(obj, rb_frame_this_func());
 }
 
-static VALUE rb_struct_ref0(obj) VALUE obj; {return RSTRUCT(obj)->ptr[0];}
-static VALUE rb_struct_ref1(obj) VALUE obj; {return RSTRUCT(obj)->ptr[1];}
-static VALUE rb_struct_ref2(obj) VALUE obj; {return RSTRUCT(obj)->ptr[2];}
-static VALUE rb_struct_ref3(obj) VALUE obj; {return RSTRUCT(obj)->ptr[3];}
-static VALUE rb_struct_ref4(obj) VALUE obj; {return RSTRUCT(obj)->ptr[4];}
-static VALUE rb_struct_ref5(obj) VALUE obj; {return RSTRUCT(obj)->ptr[5];}
-static VALUE rb_struct_ref6(obj) VALUE obj; {return RSTRUCT(obj)->ptr[6];}
-static VALUE rb_struct_ref7(obj) VALUE obj; {return RSTRUCT(obj)->ptr[7];}
-static VALUE rb_struct_ref8(obj) VALUE obj; {return RSTRUCT(obj)->ptr[8];}
-static VALUE rb_struct_ref9(obj) VALUE obj; {return RSTRUCT(obj)->ptr[9];}
+static VALUE rb_struct_ref0(VALUE obj) {return RSTRUCT_PTR(obj)[0];}
+static VALUE rb_struct_ref1(VALUE obj) {return RSTRUCT_PTR(obj)[1];}
+static VALUE rb_struct_ref2(VALUE obj) {return RSTRUCT_PTR(obj)[2];}
+static VALUE rb_struct_ref3(VALUE obj) {return RSTRUCT_PTR(obj)[3];}
+static VALUE rb_struct_ref4(VALUE obj) {return RSTRUCT_PTR(obj)[4];}
+static VALUE rb_struct_ref5(VALUE obj) {return RSTRUCT_PTR(obj)[5];}
+static VALUE rb_struct_ref6(VALUE obj) {return RSTRUCT_PTR(obj)[6];}
+static VALUE rb_struct_ref7(VALUE obj) {return RSTRUCT_PTR(obj)[7];}
+static VALUE rb_struct_ref8(VALUE obj) {return RSTRUCT_PTR(obj)[8];}
+static VALUE rb_struct_ref9(VALUE obj) {return RSTRUCT_PTR(obj)[9];}
 
-static VALUE (*ref_func[10])() = {
+#define N_REF_FUNC (sizeof(ref_func) / sizeof(ref_func[0]))
+
+static VALUE (*ref_func[])(VALUE) = {
     rb_struct_ref0,
     rb_struct_ref1,
     rb_struct_ref2,
@@ -147,8 +141,7 @@ static VALUE (*ref_func[10])() = {
 };
 
 static void
-rb_struct_modify(s)
-    VALUE s;
+rb_struct_modify(VALUE s)
 {
     if (OBJ_FROZEN(s)) rb_error_frozen("Struct");
     if (!OBJ_TAINTED(s) && rb_safe_level() >= 4)
@@ -156,30 +149,26 @@ rb_struct_modify(s)
 }
 
 static VALUE
-rb_struct_set(obj, val)
-    VALUE obj, val;
+rb_struct_set(VALUE obj, VALUE val)
 {
     VALUE members, slot;
-    ID id;
     long i;
 
     members = rb_struct_members(obj);
     rb_struct_modify(obj);
-    id = ruby_frame->orig_func;
-    for (i=0; i<RARRAY(members)->len; i++) {
-	slot = RARRAY(members)->ptr[i];
-	if (rb_id_attrset(SYM2ID(slot)) == id) {
-	    return RSTRUCT(obj)->ptr[i] = val;
+    for (i=0; i<RARRAY_LEN(members); i++) {
+	slot = RARRAY_PTR(members)[i];
+	if (rb_id_attrset(SYM2ID(slot)) == rb_frame_this_func()) {
+	    return RSTRUCT_PTR(obj)[i] = val;
 	}
     }
-    rb_name_error(ruby_frame->last_func, "`%s' is not a struct member",
-		  rb_id2name(id));
+    rb_name_error(rb_frame_this_func(), "`%s' is not a struct member",
+		  rb_id2name(rb_frame_this_func()));
     return Qnil;		/* not reached */
 }
 
 static VALUE
-make_struct(name, members, klass)
-    VALUE name, members, klass;
+make_struct(VALUE name, VALUE members, VALUE klass)
 {
     VALUE nstr;
     ID id;
@@ -193,6 +182,7 @@ make_struct(name, members, klass)
     }
     else {
 	char *cname = StringValuePtr(name);
+
 	id = rb_intern(cname);
 	if (!rb_is_const_id(id)) {
 	    rb_name_error(id, "identifier %s needs to be constant", cname);
@@ -203,17 +193,17 @@ make_struct(name, members, klass)
 	}
 	nstr = rb_define_class_under(klass, rb_id2name(id), klass);
     }
-    rb_iv_set(nstr, "__size__", LONG2NUM(RARRAY(members)->len));
+    rb_iv_set(nstr, "__size__", LONG2NUM(RARRAY_LEN(members)));
     rb_iv_set(nstr, "__members__", members);
 
     rb_define_alloc_func(nstr, struct_alloc);
     rb_define_singleton_method(nstr, "new", rb_class_new_instance, -1);
     rb_define_singleton_method(nstr, "[]", rb_class_new_instance, -1);
     rb_define_singleton_method(nstr, "members", rb_struct_s_members_m, 0);
-    for (i=0; i< RARRAY(members)->len; i++) {
-	ID id = SYM2ID(RARRAY(members)->ptr[i]);
+    for (i=0; i< RARRAY_LEN(members); i++) {
+	ID id = SYM2ID(RARRAY_PTR(members)[i]);
 	if (rb_is_local_id(id) || rb_is_const_id(id)) {
-	    if (i<10) {
+	    if (i < N_REF_FUNC) {
 		rb_define_method_id(nstr, id, ref_func[i], 0);
 	    }
 	    else {
@@ -226,22 +216,52 @@ make_struct(name, members, klass)
     return nstr;
 }
 
-#ifdef HAVE_STDARG_PROTOTYPES
-#include <stdarg.h>
-#define va_init_list(a,b) va_start(a,b)
-#else
-#include <varargs.h>
-#define va_init_list(a,b) va_start(a)
-#endif
+VALUE
+rb_struct_alloc_noinit(VALUE klass)
+{
+    return struct_alloc(klass);
+}
+
+VALUE
+rb_struct_define_without_accessor(char *class_name, VALUE super, rb_alloc_func_t alloc, ...)
+{
+    VALUE klass;
+    va_list ar;
+    VALUE members;
+    long i;
+    char *name;
+
+    members = rb_ary_new2(0);
+    va_start(ar, alloc);
+    i = 0;
+    while ((name = va_arg(ar, char*)) != NULL) {
+        rb_ary_push(members, ID2SYM(rb_intern(name)));
+    }
+    va_end(ar);
+    OBJ_FREEZE(members);
+
+    if (class_name) {
+        klass = rb_define_class(class_name, super);
+    }
+    else {
+	klass = rb_class_new(super);
+	rb_make_metaclass(klass, RBASIC(super)->klass);
+	rb_class_inherited(super, klass);
+    }
+
+    rb_iv_set(klass, "__size__", LONG2NUM(RARRAY_LEN(members)));
+    rb_iv_set(klass, "__members__", members);
+
+    if (alloc)
+        rb_define_alloc_func(klass, alloc);
+    else
+        rb_define_alloc_func(klass, struct_alloc);
+
+    return klass;
+}
 
 VALUE
-#ifdef HAVE_STDARG_PROTOTYPES
 rb_struct_define(const char *name, ...)
-#else
-rb_struct_define(name, va_alist)
-    const char *name;
-    va_dcl
-#endif
 {
     va_list ar;
     VALUE nm, ary;
@@ -251,7 +271,7 @@ rb_struct_define(name, va_alist)
     else nm = rb_str_new2(name);
     ary = rb_ary_new();
 
-    va_init_list(ar, name);
+    va_start(ar, name);
     while ((mem = va_arg(ar, char*)) != 0) {
 	ID slot = rb_intern(mem);
 	rb_ary_push(ary, ID2SYM(slot));
@@ -296,10 +316,7 @@ rb_struct_define(name, va_alist)
  */
 
 static VALUE
-rb_struct_s_def(argc, argv, klass)
-    int argc;
-    VALUE *argv;
-    VALUE klass;
+rb_struct_s_def(int argc, VALUE *argv, VALUE klass)
 {
     VALUE name, rest;
     long i;
@@ -307,18 +324,13 @@ rb_struct_s_def(argc, argv, klass)
     ID id;
 
     rb_scan_args(argc, argv, "1*", &name, &rest);
-    for (i=0; i<RARRAY(rest)->len; i++) {
-	id = rb_to_id(RARRAY(rest)->ptr[i]);
-	RARRAY(rest)->ptr[i] = ID2SYM(id);
+    for (i=0; i<RARRAY_LEN(rest); i++) {
+	id = rb_to_id(RARRAY_PTR(rest)[i]);
+	RARRAY_PTR(rest)[i] = ID2SYM(id);
     }
-    if (!NIL_P(name)) {
-	VALUE tmp = rb_check_string_type(name);
-
-	if (NIL_P(tmp)) {
-	    id = rb_to_id(name);
-	    rb_ary_unshift(rest, ID2SYM(id));
-	    name = Qnil;
-	}
+    if (!NIL_P(name) && SYMBOL_P(name)) {
+	rb_ary_unshift(rest, name);
+	name = Qnil;
     }
     st = make_struct(name, rest, klass);
     if (rb_block_given_p()) {
@@ -331,9 +343,8 @@ rb_struct_s_def(argc, argv, klass)
 /*
  */
 
-static VALUE
-rb_struct_initialize(self, values)
-    VALUE self, values;
+VALUE
+rb_struct_initialize(VALUE self, VALUE values)
 {
     VALUE klass = rb_obj_class(self);
     VALUE size;
@@ -342,20 +353,19 @@ rb_struct_initialize(self, values)
     rb_struct_modify(self);
     size = rb_struct_iv_get(klass, "__size__");
     n = FIX2LONG(size);
-    if (n < RARRAY(values)->len) {
+    if (n < RARRAY_LEN(values)) {
 	rb_raise(rb_eArgError, "struct size differs");
     }
-    MEMCPY(RSTRUCT(self)->ptr, RARRAY(values)->ptr, VALUE, RARRAY(values)->len);
-    if (n > RARRAY(values)->len) {
-	rb_mem_clear(RSTRUCT(self)->ptr+RARRAY(values)->len,
-		     n-RARRAY(values)->len);
+    MEMCPY(RSTRUCT_PTR(self), RARRAY_PTR(values), VALUE, RARRAY_LEN(values));
+    if (n > RARRAY_LEN(values)) {
+	rb_mem_clear(RSTRUCT_PTR(self)+RARRAY_LEN(values),
+		     n-RARRAY_LEN(values));
     }
     return Qnil;
 }
 
 static VALUE
-struct_alloc(klass)
-    VALUE klass;
+struct_alloc(VALUE klass)
 {
     VALUE size;
     long n;
@@ -365,28 +375,28 @@ struct_alloc(klass)
     size = rb_struct_iv_get(klass, "__size__");
     n = FIX2LONG(size);
 
-    st->ptr = ALLOC_N(VALUE, n);
-    rb_mem_clear(st->ptr, n);
-    st->len = n;
+    if (0 < n && n <= RSTRUCT_EMBED_LEN_MAX) {
+        RBASIC(st)->flags &= ~RSTRUCT_EMBED_LEN_MASK;
+        RBASIC(st)->flags |= n << RSTRUCT_EMBED_LEN_SHIFT;
+	rb_mem_clear(st->as.ary, n);
+    }
+    else {
+	st->as.heap.ptr = ALLOC_N(VALUE, n);
+	rb_mem_clear(st->as.heap.ptr, n);
+	st->as.heap.len = n;
+    }
 
     return (VALUE)st;
 }
 
 VALUE
-rb_struct_alloc(klass, values)
-    VALUE klass, values;
+rb_struct_alloc(VALUE klass, VALUE values)
 {
-    return rb_class_new_instance(RARRAY(values)->len, RARRAY(values)->ptr, klass);
+    return rb_class_new_instance(RARRAY_LEN(values), RARRAY_PTR(values), klass);
 }
 
 VALUE
-#ifdef HAVE_STDARG_PROTOTYPES
 rb_struct_new(VALUE klass, ...)
-#else
-rb_struct_new(klass, va_alist)
-    VALUE klass;
-    va_dcl
-#endif
 {
     VALUE sz, *mem;
     long size, i;
@@ -395,7 +405,7 @@ rb_struct_new(klass, va_alist)
     sz = rb_struct_iv_get(klass, "__size__");
     size = FIX2LONG(sz); 
     mem = ALLOCA_N(VALUE, size);
-    va_init_list(args, klass);
+    va_start(args, klass);
     for (i=0; i<size; i++) {
 	mem[i] = va_arg(args, VALUE);
     }
@@ -423,13 +433,13 @@ rb_struct_new(klass, va_alist)
  */
 
 static VALUE
-rb_struct_each(s)
-    VALUE s;
+rb_struct_each(VALUE s)
 {
     long i;
 
-    for (i=0; i<RSTRUCT(s)->len; i++) {
-	rb_yield(RSTRUCT(s)->ptr[i]);
+    RETURN_ENUMERATOR(s, 0, 0);
+    for (i=0; i<RSTRUCT_LEN(s); i++) {
+	rb_yield(RSTRUCT_PTR(s)[i]);
     }
     return s;
 }
@@ -453,50 +463,54 @@ rb_struct_each(s)
  */
 
 static VALUE
-rb_struct_each_pair(s)
-    VALUE s;
+rb_struct_each_pair(VALUE s)
 {
     VALUE members;
     long i;
 
+    RETURN_ENUMERATOR(s, 0, 0);
     members = rb_struct_members(s);
-    for (i=0; i<RSTRUCT(s)->len; i++) {
-	rb_yield_values(2, rb_ary_entry(members, i), RSTRUCT(s)->ptr[i]);
+    for (i=0; i<RSTRUCT_LEN(s); i++) {
+	rb_yield_values(2, rb_ary_entry(members, i), RSTRUCT_PTR(s)[i]);
     }
     return s;
 }
 
 static VALUE
-inspect_struct(s)
-    VALUE s;
+inspect_struct(VALUE s, VALUE dummy, int recur)
 {
     char *cname = rb_class2name(rb_obj_class(s));
     VALUE str, members;
     long i;
 
+    if (recur) {
+	return rb_sprintf("#<struct %s:...>", cname);
+    }
+
     members = rb_struct_members(s);
-    str = rb_str_buf_new2("#<struct ");
-    rb_str_cat2(str, cname);
-    rb_str_cat2(str, " ");
-    for (i=0; i<RSTRUCT(s)->len; i++) {
+    if (cname[0] == '#') {
+	str = rb_str_new2("#<struct ");
+    }
+    else {
+	str = rb_sprintf("#<struct %s ", cname);
+    }
+    for (i=0; i<RSTRUCT_LEN(s); i++) {
 	VALUE slot;
 	ID id;
-	char *p;
 
 	if (i > 0) {
 	    rb_str_cat2(str, ", ");
 	}
-	slot = RARRAY(members)->ptr[i];
+	slot = RARRAY_PTR(members)[i];
 	id = SYM2ID(slot);
 	if (rb_is_local_id(id) || rb_is_const_id(id)) {
-	    p = rb_id2name(id);
-	    rb_str_cat2(str, p);
+	    rb_str_append(str, rb_id2str(id));
 	}
 	else {
 	    rb_str_append(str, rb_inspect(slot));
 	}
 	rb_str_cat2(str, "=");
-	rb_str_append(str, rb_inspect(RSTRUCT(s)->ptr[i]));
+	rb_str_append(str, rb_inspect(RSTRUCT_PTR(s)[i]));
     }
     rb_str_cat2(str, ">");
     OBJ_INFECT(str, s);
@@ -513,19 +527,9 @@ inspect_struct(s)
  */
 
 static VALUE
-rb_struct_inspect(s)
-    VALUE s;
+rb_struct_inspect(VALUE s)
 {
-    if (rb_inspecting_p(s)) {
-	char *cname = rb_class2name(rb_obj_class(s));
-	size_t len = strlen(cname) + 14;
-	VALUE str = rb_str_new(0, len);
-
-	snprintf(RSTRING(str)->ptr, len+1, "#<struct %s:...>", cname);
-	RSTRING(str)->len = strlen(RSTRING(str)->ptr);
-	return str;
-    }
-    return rb_protect_inspect(inspect_struct, s, 0);
+    return rb_exec_recursive(inspect_struct, s, 0);
 }
 
 /*
@@ -541,43 +545,39 @@ rb_struct_inspect(s)
  */
 
 static VALUE
-rb_struct_to_a(s)
-    VALUE s;
+rb_struct_to_a(VALUE s)
 {
-    return rb_ary_new4(RSTRUCT(s)->len, RSTRUCT(s)->ptr);
+    return rb_ary_new4(RSTRUCT_LEN(s), RSTRUCT_PTR(s));
 }
 
 /* :nodoc: */
 static VALUE
-rb_struct_init_copy(copy, s)
-    VALUE copy, s;
+rb_struct_init_copy(VALUE copy, VALUE s)
 {
     if (copy == s) return copy;
     rb_check_frozen(copy);
     if (!rb_obj_is_instance_of(s, rb_obj_class(copy))) {
 	rb_raise(rb_eTypeError, "wrong argument class");
     }
-    if (RSTRUCT(copy)->len != RSTRUCT(s)->len) {
+    if (RSTRUCT_LEN(copy) != RSTRUCT_LEN(s)) {
 	rb_raise(rb_eTypeError, "struct size mismatch");
     }
-    MEMCPY(RSTRUCT(copy)->ptr, RSTRUCT(s)->ptr, VALUE, RSTRUCT(copy)->len);
+    MEMCPY(RSTRUCT_PTR(copy), RSTRUCT_PTR(s), VALUE, RSTRUCT_LEN(copy));
 
     return copy;
 }
 
 static VALUE
-rb_struct_aref_id(s, id)
-    VALUE s;
-    ID id;
+rb_struct_aref_id(VALUE s, ID id)
 {
     VALUE members;
     long i, len;
 
     members = rb_struct_members(s);
-    len = RARRAY(members)->len;
+    len = RARRAY_LEN(members);
     for (i=0; i<len; i++) {
-	if (SYM2ID(RARRAY(members)->ptr[i]) == id) {
-	    return RSTRUCT(s)->ptr[i];
+	if (SYM2ID(RARRAY_PTR(members)[i]) == id) {
+	    return RSTRUCT_PTR(s)[i];
 	}
     }
     rb_name_error(id, "no member '%s' in struct", rb_id2name(id));
@@ -604,8 +604,7 @@ rb_struct_aref_id(s, id)
  */
 
 VALUE
-rb_struct_aref(s, idx)
-    VALUE s, idx;
+rb_struct_aref(VALUE s, VALUE idx)
 {
     long i;
 
@@ -614,34 +613,32 @@ rb_struct_aref(s, idx)
     }
 
     i = NUM2LONG(idx);
-    if (i < 0) i = RSTRUCT(s)->len + i;
+    if (i < 0) i = RSTRUCT_LEN(s) + i;
     if (i < 0)
         rb_raise(rb_eIndexError, "offset %ld too small for struct(size:%ld)",
-		 i, RSTRUCT(s)->len);
-    if (RSTRUCT(s)->len <= i)
+		 i, RSTRUCT_LEN(s));
+    if (RSTRUCT_LEN(s) <= i)
         rb_raise(rb_eIndexError, "offset %ld too large for struct(size:%ld)",
-		 i, RSTRUCT(s)->len);
-    return RSTRUCT(s)->ptr[i];
+		 i, RSTRUCT_LEN(s));
+    return RSTRUCT_PTR(s)[i];
 }
 
 static VALUE
-rb_struct_aset_id(s, id, val)
-    VALUE s, val;
-    ID id;
+rb_struct_aset_id(VALUE s, ID id, VALUE val)
 {
     VALUE members;
     long i, len;
 
     members = rb_struct_members(s);
     rb_struct_modify(s);
-    len = RARRAY(members)->len;
-    if (RSTRUCT(s)->len != RARRAY(members)->len) {
-	rb_raise(rb_eTypeError, "struct size differs (%d required %d given)",
-		 RARRAY(members)->len, RSTRUCT(s)->len);
+    len = RARRAY_LEN(members);
+    if (RSTRUCT_LEN(s) != RARRAY_LEN(members)) {
+	rb_raise(rb_eTypeError, "struct size differs (%ld required %ld given)",
+		 RARRAY_LEN(members), RSTRUCT_LEN(s));
     }
     for (i=0; i<len; i++) {
-	if (SYM2ID(RARRAY(members)->ptr[i]) == id) {
-	    RSTRUCT(s)->ptr[i] = val;
+	if (SYM2ID(RARRAY_PTR(members)[i]) == id) {
+	    RSTRUCT_PTR(s)[i] = val;
 	    return val;
 	}
     }
@@ -670,8 +667,7 @@ rb_struct_aset_id(s, id, val)
  */
 
 VALUE
-rb_struct_aset(s, idx, val)
-    VALUE s, idx, val;
+rb_struct_aset(VALUE s, VALUE idx, VALUE val)
 {
     long i;
 
@@ -680,24 +676,21 @@ rb_struct_aset(s, idx, val)
     }
 
     i = NUM2LONG(idx);
-    if (i < 0) i = RSTRUCT(s)->len + i;
+    if (i < 0) i = RSTRUCT_LEN(s) + i;
     if (i < 0) {
         rb_raise(rb_eIndexError, "offset %ld too small for struct(size:%ld)",
-		 i, RSTRUCT(s)->len);
+		 i, RSTRUCT_LEN(s));
     }
-    if (RSTRUCT(s)->len <= i) {
+    if (RSTRUCT_LEN(s) <= i) {
         rb_raise(rb_eIndexError, "offset %ld too large for struct(size:%ld)",
-		 i, RSTRUCT(s)->len);
+		 i, RSTRUCT_LEN(s));
     }
     rb_struct_modify(s);
-    return RSTRUCT(s)->ptr[i] = val;
+    return RSTRUCT_PTR(s)[i] = val;
 }
 
-static VALUE struct_entry _((VALUE, long));
 static VALUE
-struct_entry(s, n)
-    VALUE s;
-    long n;
+struct_entry(VALUE s, long n)
 {
     return rb_struct_aref(s, LONG2NUM(n));
 }
@@ -719,12 +712,9 @@ struct_entry(s, n)
  */
 
 static VALUE
-rb_struct_values_at(argc, argv, s)
-    int argc;
-    VALUE *argv;
-    VALUE s;
+rb_struct_values_at(int argc, VALUE *argv, VALUE s)
 {
-    return rb_values_at(s, RSTRUCT(s)->len, argc, argv, struct_entry);
+    return rb_get_values_at(s, RSTRUCT_LEN(s), argc, argv, struct_entry);
 }
 
 /*
@@ -742,10 +732,7 @@ rb_struct_values_at(argc, argv, s)
  */
 
 static VALUE
-rb_struct_select(argc, argv, s)
-    int argc;
-    VALUE *argv;
-    VALUE s;
+rb_struct_select(int argc, VALUE *argv, VALUE s)
 {
     VALUE result;
     long i;
@@ -754,9 +741,9 @@ rb_struct_select(argc, argv, s)
 	rb_raise(rb_eArgError, "wrong number of arguments (%d for 0)", argc);
     }
     result = rb_ary_new();
-    for (i = 0; i < RSTRUCT(s)->len; i++) {
-	if (RTEST(rb_yield(RSTRUCT(s)->ptr[i]))) {
-	    rb_ary_push(result, RSTRUCT(s)->ptr[i]);
+    for (i = 0; i < RSTRUCT_LEN(s); i++) {
+	if (RTEST(rb_yield(RSTRUCT_PTR(s)[i]))) {
+	    rb_ary_push(result, RSTRUCT_PTR(s)[i]);
 	}
     }
 
@@ -781,20 +768,19 @@ rb_struct_select(argc, argv, s)
  */
 
 static VALUE
-rb_struct_equal(s, s2)
-    VALUE s, s2;
+rb_struct_equal(VALUE s, VALUE s2)
 {
     long i;
 
     if (s == s2) return Qtrue;
     if (TYPE(s2) != T_STRUCT) return Qfalse;
     if (rb_obj_class(s) != rb_obj_class(s2)) return Qfalse;
-    if (RSTRUCT(s)->len != RSTRUCT(s2)->len) {
+    if (RSTRUCT_LEN(s) != RSTRUCT_LEN(s2)) {
 	rb_bug("inconsistent struct"); /* should never happen */
     }
 
-    for (i=0; i<RSTRUCT(s)->len; i++) {
-	if (!rb_equal(RSTRUCT(s)->ptr[i], RSTRUCT(s2)->ptr[i])) return Qfalse;
+    for (i=0; i<RSTRUCT_LEN(s); i++) {
+	if (!rb_equal(RSTRUCT_PTR(s)[i], RSTRUCT_PTR(s2)[i])) return Qfalse;
     }
     return Qtrue;
 }
@@ -807,16 +793,15 @@ rb_struct_equal(s, s2)
  */
 
 static VALUE
-rb_struct_hash(s)
-    VALUE s;
+rb_struct_hash(VALUE s)
 {
     long i, h;
     VALUE n;
 
     h = rb_hash(rb_obj_class(s));
-    for (i = 0; i < RSTRUCT(s)->len; i++) {
+    for (i = 0; i < RSTRUCT_LEN(s); i++) {
 	h = (h << 1) | (h<0 ? 1 : 0);
-	n = rb_hash(RSTRUCT(s)->ptr[i]);
+	n = rb_hash(RSTRUCT_PTR(s)[i]);
 	h ^= NUM2LONG(n);
     }
     return LONG2FIX(h);
@@ -831,20 +816,19 @@ rb_struct_hash(s)
  */
 
 static VALUE
-rb_struct_eql(s, s2)
-    VALUE s, s2;
+rb_struct_eql(VALUE s, VALUE s2)
 {
     long i;
 
     if (s == s2) return Qtrue;
     if (TYPE(s2) != T_STRUCT) return Qfalse;
     if (rb_obj_class(s) != rb_obj_class(s2)) return Qfalse;
-    if (RSTRUCT(s)->len != RSTRUCT(s2)->len) {
+    if (RSTRUCT_LEN(s) != RSTRUCT_LEN(s2)) {
 	rb_bug("inconsistent struct"); /* should never happen */
     }
 
-    for (i=0; i<RSTRUCT(s)->len; i++) {
-	if (!rb_eql(RSTRUCT(s)->ptr[i], RSTRUCT(s2)->ptr[i])) return Qfalse;
+    for (i=0; i<RSTRUCT_LEN(s); i++) {
+	if (!rb_eql(RSTRUCT_PTR(s)[i], RSTRUCT_PTR(s2)[i])) return Qfalse;
     }
     return Qtrue;
 }
@@ -862,10 +846,9 @@ rb_struct_eql(s, s2)
  */
 
 static VALUE
-rb_struct_size(s)
-    VALUE s;
+rb_struct_size(VALUE s)
 {
-    return LONG2FIX(RSTRUCT(s)->len);
+    return LONG2FIX(RSTRUCT_LEN(s));
 }
 
 /*
@@ -884,7 +867,7 @@ rb_struct_size(s)
  *  <code>Symbol</code> (such as <code>:name</code>).
  */
 void
-Init_Struct()
+Init_Struct(void)
 {
     rb_cStruct = rb_define_class("Struct", rb_cObject);
     rb_include_module(rb_cStruct, rb_mEnumerable);
